/*************************************************************************
 *************************************************************************
 **									**
 **	 		Copyright (c) 1994, 1995 			**
 **	    Brown University Institute for Medical Computing		**
 **	      and the American College of Cardiology (ACC)		**
 **									**
 ** The software and support documentation for the American College of	**
 ** Cardiology's 1995 demonstration of DICOM interchange media were	**
 ** developed at the Brown University Institute for Medical Computing.	**
 **									** 
 ** THIS SOFTWARE IS MADE AVAILABLE "AS IS", AND NEITHER THE AMERICAN	**
 ** COLLEGE OF CARDIOLOGY (ACC) NOR BROWN UNIVERSITY MAKE ANY WARRANTY	**
 ** ABOUT THE SOFTWARE, ITS PERFORMANCE, ITS MERCHANTABILITY OR FITNESS	**
 ** FOR ANY PARTICULAR USE, FREEDOM FROM DEFECTS, OR ITS CONFORMANCE TO	**
 ** ANY SPECIFICATION.  THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE	**
 ** OF THE SOFTWARE IS WITH THE USER.					**
 **									** 
 ** Copyright of the software and supporting documentation is jointly	**
 ** owned by the ACC and Brown University, and free access is hereby	**
 ** granted as a license to use this software, copy this software and	**
 ** prepare derivative works based upon this software.  However, any	**
 ** distribution of the source code or executable versions of this	**
 ** software, or of the	 supporting documentation (or of derivative	**
 ** works of the software or supporting documentation) must include	**
 ** this copyright notice and explanation.				**
 **									** 
 ** Author: Jonathan L. Elion MD, FACC					**
 **									**
 ************************************************************************* 
 *************************************************************************/ 

#define COPYRIGHT1 "\t\t\t Copyright (c) 1994, 1995"
#define COPYRIGHT2 "\t    Brown University Institute for Medical Computing"
#define COPYRIGHT3 "\t      and the American College of Cardiology (ACC)"


#define MAX_LINE_LENGTH 256	/* Max length of script input line */
#define MAX_WORDS 5		/* Max number of words on a line */
#define BUFFER_LENGTH 4096	/* Length of DICOM message buffer */
#define LITTLE_ENDIAN		0
#define BIG_ENDIAN		1
#define ENDIAN			2
#define NO_ENDIAN		3
#define PARSING			1
#define SCANNING		2

#define MAX_SYMBOLS		100 /* Maximum size of symbol table */
#define MAX_NAME_SIZE		128 /* Maximum size of file names */
#define MAX_OFFSETS		500 /* Maximum # of entries in offset table */

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#define XRAY_IMAGE_SOP_CLASS_UID	"1.2.840.10008.5.1.4.1.1.12.1"
#define US_SOP_CLASS_UID		"1.2.840.10008.5.1.4.1.1.6"
#define MULTIFRAME_US_SOP_CLASS_UID	"1.2.840.10008.5.1.4.1.1.3"
#define DICOMDIR_SOP_CLASS_UID		"1.2.840.10008.1.3.10"
#define LITTLE_ENDIAN_IMPLICIT_UID	"1.2.840.10008.1.2"
#define LITTLE_ENDIAN_EXPLICIT_UID	"1.2.840.10008.1.2.1"
#define LOSSLESS_JPEG_14_UID		"1.2.840.10008.1.2.4.70"

/*
 * NOTE: For the DOS version of this program, the name of the directory
 *	file is simply DICOMDIR.  When the program is run under Unix,
 *	the file name may have to be changed to "dicomdir."	
 */
#define DICOMDIR_NAME "dicomdir." 

/*
 * NOTE: For the DOS version of this program, the path name used to
 *       access the CDROM drive is "E:".  This may have to be changed
 *       depending on the operating system and the location of the drive
 */
#define CDROM_PATH "/cdrom/"

/*
 * NOTE: The "unsigned long" used in DICOM is 32-bits long.  Some
 *	 machines may have unsigned longs as 64-bits long.  To get
 *	 around this inconsistency, the UnsignedLong type is defined
 *	 here.  It should be left as "unsigned long" on most machines.
 *	 If your target CPU has unsigned long as 64-bit words, then
 *	 change this typedef to unsigned int (most likely).  If you
 *	 are uncertain, LEAVE IT ALONE, as the checkEndian() routine
 *	 now checks to be sure that UnsignedLong is 32-bits long.
 */
typedef unsigned long UnsignedLong;

/* DICOM Value Representations.  See Part 5, Table 6.2-1 */
typedef enum {			/* VALUE REPRESENTATION (VR) TYPES */
    VR_AE,			/*  0 - Application Entity */
    VR_AS,			/*  1 - Age string */
    VR_AT,			/*  2 - Attribute tag */
    VR_CS,			/*  3 - Control string */
    VR_DA,			/*  4 - Date */
    VR_DS,			/*  5 - Decimal string */
    VR_DT,			/*  6 - Date/Time */
    VR_FL,			/*  7 - Float */
    VR_FD,			/*  8 - Floating double */
    VR_IS,			/*  9 - Integer string */
    VR_LO,			/* 10 - Long string */
    VR_LT,			/* 11 - Long text */
    VR_OB,			/* 12 - Other, byte */
    VR_OW,			/* 13 - Other, word */
    VR_PN,			/* 14 - Person Name */
    VR_SH,			/* 15 - Short string */
    VR_SL,			/* 16 - Signed long */
    VR_SQ,			/* 17 - Sequence of items */
    VR_SS,			/* 18 - Signed short */
    VR_ST,			/* 19 - Short text */
    VR_TM,			/* 20 - Time */
    VR_UI,			/* 21 - Unique identifier (UID) */
    VR_UL,			/* 22 - Unsigned long */
    VR_US,			/* 23 - Unsigned short */
    VR_DATA,			/* 24 - Data tag */
    VR_UNKNOWN			/* 25 - Unknown/unspecified */
}   VALUE_REPRESENTATION;

#define VR_AE_LENGTH 16		/* Application Entity */
#define VR_AS_LENGTH 4		/* Age string */
#define VR_AT_LENGTH 4		/* Attribute tag */
#define VR_CS_LENGTH 16		/* Control string */
#define VR_DA_LENGTH 8		/* Date */
#define VR_DS_LENGTH 16		/* Decimal string */
#define VR_DT_LENGTH 26		/* Date/Time */
#define VR_FL_LENGTH 16		/* Float */
#define VR_FD_LENGTH 4		/* Floating double */
#define VR_IS_LENGTH 8		/* Integer string */
#define VR_LO_LENGTH 64		/* Long string */
#define VR_LT_LENGTH 10240	/* Long text */
#define VR_PN_LENGTH 64		/* Person Name */
#define VR_SH_LENGTH 16		/* Short string */
#define VR_SL_LENGTH 4		/* Signed long */
#define VR_SS_LENGTH 2		/* Signed short */
#define VR_ST_LENGTH 1024	/* Short text */
#define VR_TM_LENGTH 16		/* Time */
#define VR_UI_LENGTH 64		/* Unique identifier (UID) */
#define VR_UL_LENGTH 4		/* Unsigned long */
#define VR_US_LENGTH 2		/* Unsigned short */

/* Space reserved for strings is maximum length + 1 bytes */
typedef struct metaInfoStruct { 
    unsigned char version[2];
    char SOP_ClassUID[VR_UI_LENGTH + 1];
    char SOP_InstanceUID[VR_UI_LENGTH + 1];
    char transferSyntaxUID[VR_UI_LENGTH + 1];
    char implClassUID[VR_UI_LENGTH + 1];
    char versionName[VR_SH_LENGTH + 1];
    char appEntityTitle[VR_AE_LENGTH + 1];
    char privateInfoUID[VR_UI_LENGTH + 1];
    char *privateInfo;
} metaInfoStructName; 

typedef struct dictStruct {
    unsigned short group;
    unsigned short element;
    VALUE_REPRESENTATION VR;
    char *description;
} dictStructName;

struct dictStruct *dictLookup(unsigned short, unsigned short);
int checkEndian(void);
