/*************************************************************************
 *************************************************************************
 **									**
 **	 		Copyright (c) 1994, 1995 			**
 **	    Brown University Institute for Medical Computing		**
 **	      and the American College of Cardiology (ACC)		**
 **									**
 ** The software and support documentation for the American College of	**
 ** Cardiology's 1995 demonstration of DICOM interchange media were	**
 ** developed at the Brown University Institute for Medical Computing.	**
 **									** 
 ** THIS SOFTWARE IS MADE AVAILABLE "AS IS", AND NEITHER THE AMERICAN	**
 ** COLLEGE OF CARDIOLOGY (ACC) NOR BROWN UNIVERSITY MAKE ANY WARRANTY	**
 ** ABOUT THE SOFTWARE, ITS PERFORMANCE, ITS MERCHANTABILITY OR FITNESS	**
 ** FOR ANY PARTICULAR USE, FREEDOM FROM DEFECTS, OR ITS CONFORMANCE TO	**
 ** ANY SPECIFICATION.  THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE	**
 ** OF THE SOFTWARE IS WITH THE USER.					**
 **									** 
 ** Copyright of the software and supporting documentation is jointly	**
 ** owned by the ACC and Brown University, and free access is hereby	**
 ** granted as a license to use this software, copy this software and	**
 ** prepare derivative works based upon this software.  However, any	**
 ** distribution of the source code or executable versions of this	**
 ** software, or of the	 supporting documentation (or of derivative	**
 ** works of the software or supporting documentation) must include	**
 ** this copyright notice and explanation.				**
 **									** 
 ** Author: Jonathan L. Elion MD, FACC					**
 **									**
 ************************************************************************* 
 *************************************************************************/ 


/************************************************************************
 *									*
 * This program is intended to be the starting point for vendor 	*
 * programs to display images from the CD-R for the ACC'95 DICOM demo.	*
 * The main purpose of the program is to show how to interact with	*
 * the information contained in the DICOMDIR file.  Error checking	*
 * is minimal, as it is assumed that a reasonable DICOMDIR file is	*
 * present.  One thing in particular that is not checked is the order	*
 * of the DICOM tags (defined as being presented in ascending order).	*
 *									*
 * You are welcome to use portions of this code in your demonstration	*
 * or in commercial products, provided the copyright notice is left	*
 * intact (the copyright notice must be printed on the screen when the	*
 * program is first invoked).						*
 *									*
 * To modify this code, you will have to change the code that provides	*
 * the user interaction for the selection process (presumably, you	*
 * will want this to use the Graphical User Interface (GUI) of your	*
 * target display system).  You will also have to write code that	*
 * moves the image pixels to the display's frame buffer, and also	*
 * code to handle of the icon images (if you choose to implement this	*
 * feature).								*
 *									*
 * This program ignores many of the fields in DICOMDIR, but does	*
 * extract and store the information (rather than just skip over it).	*
 * This is intended to make it easier to modify the program to use	*
 * the information if you are interested in it.				*
 *									*
 ************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "acc95.h"
#include "demo.h"

#ifdef __BORLANDC__
#include <graphics.h>
#include <dos.h>
int graphicsDriver, graphicsMode, grErr;
#endif
unsigned char palette[256][3];


/************************************************************************
 * The following items are GLOBAL, and are used by several routines.	*
 ************************************************************************/

/* Pointer to the start of the linked list of flattened directory records */
struct dirRecord *dirList = (struct dirRecord *) NULL;

/* A single record constructed as the directory tree is traversed */
struct dirRecord thisRec;

int choiceCount; 			/* Counter used by list selection */
FILE *inFile;				/* Name of the input file */
char filesetID[VR_CS_LENGTH + 1];	/* Name of DICOM file set */
static int thisEndian;			/* "Endian"ism of host CPU */
int explicit_VR = FALSE;		/* Is input file in explicit VR mode? */
int encodedPixels = FALSE;		/* Xfer syntax has encoded pixels? */
int rowNumber;				/* Row # in image being decompressed */


/************************************************************************
 * The initGUI() routine intializes the Graphical User Interface	*
 * environment.  This is called at the beginning of the execution of	*
 * the program to set up display windows, and other routine chores	*
 * that are part of setting up the GUI environment.  Please be sure	*
 * to include proper copyright credits (and add your own, of course).	*
 ************************************************************************/

void initGUI(void)
{
    printf("------------------------------------");
    printf("------------------------------------\n");
    printf("%s\n", COPYRIGHT1);
    printf("%s\n", COPYRIGHT2);
    printf("%s\n", COPYRIGHT3);
    printf("------------------------------------");
    printf("------------------------------------\n\n");
}


/************************************************************************
 * The initList() routine is called to set up a list of choices that	*
 * the user will select from.  Typically, this would be used to set up	*
 * a selection window, initialize a pointer to the head of the		*
 * selection list, etc.
 ************************************************************************/

void initList(void)
{
    printf("File Set ID: \"%s\"\n\n", filesetID);
    printf("Choose an image to view:\n\n");
    printf(" 0. << Exit>> \n");
    choiceCount = 0;
}


/************************************************************************
 * The addToList() routine is called repeatedly, once for each image	*
 * found on the media.  Typically, the data from the dirRecord		*
 * structure would be formatted into a line suitable for inclusion in	*
 * the scrolling selection box.  The approach implemented in this demo	*
 * gives a "flattened" view of the media, namely, there is a single	*
 * list of all of the images on the media.  Alternatively, you might	*
 * want a more structured approach (pick a patient, then pick the	*
 * study, then series, then image).  This could be implemented by a	*
 * different control loop using the same information, or by changing	*
 * whole control structure of this demo program to allow interaction	*
 * and selection while parsing DICOMDIR.				*
 * example of this type of hierachical selection scheme.		*
 ************************************************************************/

void addToList(struct dirRecord *recordToAdd)
{
    printf("%2d. Patient: %s   Study: %s\n", ++choiceCount, 
	recordToAdd->patientInfo.patientName,
	recordToAdd->studyInfo.studyDescription);
    printf("    Series: %s   Image: %s   File: %s\n",
	recordToAdd->seriesInfo.seriesNumber,
	recordToAdd->imageInfo.imageNumber,
	recordToAdd->imageInfo.fileID);

    /*
     * HELPFUL HINT: recordToAdd->imageInfo.icon[128][128] is the icon for 
     * this image.  Not anything useful we can do with this in this ASCII
     * version of the demo, but it is a nice this to add to the selection
     * function.  If this program is compiled under DOS, the icons will
     * not be saved (not enough memory in DOS).
     */
}


/************************************************************************
 * The selectFromList() routine should allow the user to interact with	*
 * the list and make a selection.  The routine should return a pointer	*
 * the the directory record selected (or NULL if the user wants to exit	*
 * the program).  At a demonstration at a show, you probably do not	*
 * want to allow the user to exit (at least, no so easily!).		*
 ************************************************************************/

struct dirRecord *selectFromList(void)
{
    int i, choice; /* for loops */
    char answer[80]; /* User's typed-in answer */
    struct dirRecord *oneRecord;

    printf("\n");
    do {
	printf("Choice==> ");
	gets(answer);
	sscanf(answer, "%d", &choice);
	if ((choice > choiceCount) || (choice < 0))
	    printf("Enter a selection from 0 to %d\n", choiceCount);
    } while ((choice > choiceCount) || (choice < 0));
    if (choice == 0) return (struct dirRecord *) NULL;

    oneRecord = dirList;
    if (choice > 1) for (i=1; i<choice; i++) oneRecord = oneRecord->nextRecord;
    return oneRecord;
}


/************************************************************************
 *									*
 * The badFile() routine is used to bail out from reading the DICOMDIR	*
 * file when an error is found.  An error message is printed, and	*
 * the program exits.  Simple.						*
 *									*
 ************************************************************************/

void badFile(char *msg)
{
    fprintf(stderr, "%s\n", msg);
    exit(EXIT_FAILURE);
}


/************************************************************************
 *									*
 * The readBytes() routine is used to read bytes from DICOMDIR.  It	*
 * is passed a pointer to the destiniation for the data, the length	*
 * of the data field to read, and a flag indicating whether an		*
 * adjustment for "Endian"ism needs to be made.  If this flag is set	*
 * to NO_ENDIAN, or if the code is run on a LittleEndian machine,	*
 * then the bytes are read in directly.  If, however, the flag is set	*
 * to ENDIAN and this is NOT a LittleEndian machine, then the byte	*
 * order is reversed.							*
 *									*
 ************************************************************************/

void readBytes(char *value, long length, int flag)
{
    int i;

    /*
     * If don't care about ENDIANism, or if running on LITTLE_ENDIAN,
     * the bytes can be read directly, otherwise, the byte order has
     * to be reversed.  This makes the assumption that DICOMDIR is
     * stored in LITTLE_ENDIAN format.
     */
    if ((thisEndian == LITTLE_ENDIAN) || (flag == NO_ENDIAN))
    {
	if (fread(value, length, 1, inFile) == 1) return;
	else badFile("Unexpected EOF in input file");
    }
    else for (i = length-1; i >= 0; i--)
	if (fread(value + i, 1, 1, inFile) != 1)
	    badFile("Unexpected EOF in input file");
    return;
}


/************************************************************************
 *									*
 * The readTag() routine is simply a short-hand for reading the group,	*
 * element, and length fields from a DICOM file.  The mode for reading	*
 * VR is also checked to see if it is IMPLICIT or EXPLICIT.  The VR	*
 * field is basically ignored for this demonstration program, and the	*
 * VR, if any, is skipped.  The length field is read correctly, based	*
 * again on the EXPLICIT or IMPLICIT VR mode (length fields may be	*
 * 2 bytes in EXPLICIT VR mode, and 4 bytes in IMPLICIT VR mode).  The	*
 * routine (as of the beta release) returns the number of bytes read.	*
 *									*
 ************************************************************************/

int readTag(unsigned short *group, unsigned short *element,
    UnsignedLong *length) 
{
    char VR_Tag[2];
    unsigned short shortValue;
    int bytesRead;

    readBytes((char *) group, 2, ENDIAN);
    readBytes((char *) element, 2, ENDIAN);
    bytesRead = 8;

    if (((explicit_VR == TRUE) || (*group == 0x0002)) && (*group != 0xfffe))
    {
	readBytes(VR_Tag, 2, NO_ENDIAN);
	if ((strncmp(VR_Tag, "OB", 2) == 0) ||
	    (strncmp(VR_Tag, "OW", 2) == 0) ||
	    (strncmp(VR_Tag, "SQ", 2) == 0))
	{
	    readBytes((char *) &shortValue, 2, ENDIAN);
	    readBytes((char *) length, 4, ENDIAN);
	    bytesRead += 4;
	}
	else
	{
	    readBytes((char *) &shortValue, 2, ENDIAN);
	    *length = shortValue; 
	}
    }
    else readBytes((char *) length,  4, ENDIAN);
    return bytesRead;
}


/************************************************************************
 *									*
 * The readString() routine reads in strings that are NOT UID's and	*
 * sets the string termination (removing the trailing blank, if any).	*
 * UID's don't need to be read with this routine, as they are already	*
 * padded with a 0x00 if they are not even length.			*
 *									*
 ************************************************************************/

void readString(char *value, long length)
{
    readBytes(value, length, NO_ENDIAN);
    if (value[length-1] == ' ') value[length-1] = '\0';
    else value[length] = '\0';
}


/************************************************************************
 *									*
 * The readFileMetaInfo() routine reads the DICOM File Meta Information	*
 * from the input file.  The content of the File Meta Information is	*
 * defined in Part 10, table 7.1.1.  After the 128-byte	preamble and	*
 * the 4-byte prefix are read, the remaining portion of	the File Meta	*
 * Information Header is read assuming the Explicit VR Little Endian	*
 * Transfer Syntax (was initially defined in Part 5 as Implicit; the	*
 * change to Explicit was voted in September/October, 1994).		*
 *									*
 * NOTE: Even the DICOMDIR itself is consider a DICOM file, and as	*
 * such, must have its own File Meta Information header.		*
 *									*
 ************************************************************************/

void readFileMetaInfo(struct metaInfoStruct *metaInfo)
{
    unsigned short group,		/* Group data from tag */
		element;		/* Element data from tag */
    UnsignedLong length,		/* Length field from tag */
    		groupLength = 9999;	/* Length of group 0x0002 data */
    char	preamble[128],		/* Holds meta info preamble */
		*privateInfo;		/* Points to malloc'd private data */
    int		bytesRead;		/* Helps keep track of group length */

    /* ----- Check the DICOM File Preamble ----- */
    readBytes(preamble, 128, NO_ENDIAN);
    readBytes(preamble, 4, NO_ENDIAN);
    if (strncmp(preamble, "DICM", 4) != 0) badFile("Not a valid DICOM file");

    /*
     * Read the File Meta Information.  This is done using the Group Length
     * as an indicator of how much data to read.  This length value is set
     * initially at 9999, but quickly gets set to the correct length (the
     * length recorded in the File Meta Information segment is the length of
     * all Meta Information AFTER the Group Length).
     */
    memset((char *) metaInfo, '\0', sizeof(struct metaInfoStruct));
    while (groupLength > 0) /* Loop while there's more data */
    {
	bytesRead = readTag(&group, &element, &length);
	if (group != 2) badFile("Invalid File Meta Information");
	groupLength -= bytesRead + length; /* Decrement # of bytes remaining */
	switch(element) /* Handle each tag accordingly */
	{
	    case 0x0000: /* Group Length */
		readBytes((char *) &groupLength, length, ENDIAN);
		break;
	    case 0x0001: /* Meta Information Version Number */
		readBytes((char *) metaInfo->version, length, NO_ENDIAN);
		break; 
	    case 0x0002: /* SOP Class UID */
		readBytes(metaInfo->SOP_ClassUID, length, NO_ENDIAN);
		break;
	    case 0x0003: /* SOP Instance UID */
		readBytes(metaInfo->SOP_InstanceUID,length, NO_ENDIAN);
		break; 
	    case 0x0010: /* Transfer Syntax */
		readBytes(metaInfo->transferSyntaxUID,length, NO_ENDIAN);
		break; 
	    case 0x0012: /* Implementation Class UID */
		readBytes(metaInfo->implClassUID, length, NO_ENDIAN);
		break; 
	    case 0x0013: /* Version Name */
		readString(metaInfo->versionName, length);
		break; 
	    case 0x0016: /* Application Entity Title */
		readString(metaInfo->appEntityTitle, length);
		break; 
	    case 0x0100: /* Private Information UID */
		readBytes(metaInfo->privateInfoUID, length, NO_ENDIAN);
		break; 
	    case 0x0102: /* Private Information */
		privateInfo = malloc(length);
		if (metaInfo->privateInfo == NULL)
		    badFile("Insufficient memory for private information");
		readBytes((char *) privateInfo, length, NO_ENDIAN);
		metaInfo->privateInfo = privateInfo;
		break; 
	    default:
		badFile("Invalid data in File Meta Information");
	} /* switch(element) */
    } /* While there's still more Meta Information to read */
}


/************************************************************************
 *									*
 * The displayRow() routine is the interface to the public domain JPEG	*
 * routines.  The routine is called from the decompFrame() routine(s)	*
 * when a row of pixels has been decompressed and is ready for display.	*
 * While this program structure is not optimal, it did allow the use	*
 * of the JPEG routines with very little modification.			*
 *									*
 ************************************************************************/

void displayRow(unsigned char *row)
{
    int i;
/* Use these values for 640 x 480 screen:
    const int firstRow = 16, lastRow = 495;
    const int xoffset = 64, yoffset = 0;
*/
/* Use these values for 800 x 600 screen: */
    const int firstRow = 0, lastRow = 512;
    const int xoffset = 150, yoffset = 42;

    if ((rowNumber < firstRow) || (rowNumber > lastRow))
    {
	/* printf("Skipping row %d\n", rowNumber); */
    }
    else
    {
#ifdef __BORLANDC__
	for (i=0; i<512; i++) putpixel(i+xoffset, rowNumber+yoffset, row[i]);
#endif
	/* printf("Displaying row %d\n", rowNumber); */
    }
    rowNumber++;
}


/************************************************************************
 *									*
 * The displayImage() routine is used to display a multi-frame image.	*
 * The version that is provided here is somewhat of a "bare shell" of	*
 * a routine, as the types of things that might be done with the image	*
 * is highly vendor-specific.  This routine shows how information can	*
 * be parsed from the IOD file, and how frames in the multi-frame	*
 * image can be isolated.						*
 *									*
 ************************************************************************/

void displayImage(struct dirRecord *oneRecord)
{
    struct metaInfoStruct metaInfo;	/* IOD File Meta Information */
    unsigned short group, element,	/* Used to read IOD attributes */
	planar,				/* RGB is planar or interleaved */
	rows, columns;			/* Image size (rows and columns) */
    UnsignedLong length,		/* Used to read IOD length fields */
	offsetTable[MAX_OFFSETS],	/* frame offsets in pixel data */
	pixelStart;			/* Start of pixel stream */
    char stringBuffer[80],		/* Used to read strings from IOD */
	photometric[32],		/* Photometric interpretation */
	fileName[32];			/* Name of file to open */
    float frameTime;			/* Time between frames */
    int frameCount,			/* Number of frames in image */
	i, j,				/* Loop counters */
	warn,
	displayCount;			/* Number of frames to display */
    static unsigned char pixelBuffer[640 * 3];

    /* Attempt to open the IOD file pointed to by user's selection */
    printf("Image is in file \"%s\"\n", oneRecord->imageInfo.fileID);
    strcpy(fileName, CDROM_PATH);
    strcat(fileName, oneRecord->imageInfo.fileID);
    strcat(fileName, ".");
    inFile = fopen(fileName, "rb");
    if (inFile == NULL)
    {
	printf("Unable to open file %s\n", fileName);
	return;
    }

    /*
     * Read the file meta information, and see if the file is IOD that
     * we recognize.  If not, just ignore it.
     */
    readFileMetaInfo(&metaInfo);
    if (strcmp(metaInfo.SOP_ClassUID, XRAY_IMAGE_SOP_CLASS_UID) == 0)
    {
	printf("Image file is an X-Ray Angiographic object\n");
    }
    else if (strcmp(metaInfo.SOP_ClassUID, US_SOP_CLASS_UID) == 0)
    {
	printf("Image file is a single-frame ultrasound object\n");
	frameCount = 1;
    }
    else if (strcmp(metaInfo.SOP_ClassUID, MULTIFRAME_US_SOP_CLASS_UID) == 0)
    {
	printf("Image file is a multi-frame ultrasound object\n");
    }
    else
    {
	printf("Don't know how to display SOP Class %s\n",
	    metaInfo.SOP_ClassUID);
	return;
    }

    /*
     * Determine Explicit/Implicit VR status (should be Explicit) and
     * whether pixels are encoded (this MUST be true to comply with the
     * standard, but this allows pixels to be uncompressed during initial
     * testing).  
     */
    if (strcmp(metaInfo.transferSyntaxUID, LITTLE_ENDIAN_IMPLICIT_UID) !=0)
    {
	explicit_VR = TRUE;
	printf("*** Switching to explicit VR mode: %s ***\n",
	    metaInfo.transferSyntaxUID);
	if (strcmp(metaInfo.transferSyntaxUID, LOSSLESS_JPEG_14_UID) == 0)
	    encodedPixels = TRUE;
	else encodedPixels = FALSE;
    }

    /*
     * Now read attributes from the IOD, examining interesting ones.
     * For this demo version of the software, only a few of the attrributes
     * are checked.  For product-level code, a more comprehensive set of
     * attributes should be handled.
     */
    do {
	readTag(&group, &element, &length);
	if ((group == 0x0018) && (element == 0x1063)) /* Frame Time */
	{
	    readString(stringBuffer, length); /* decimal string */
	    sscanf(stringBuffer, "%f", &frameTime);
	    printf("%f milliseconds between frames\n", frameTime);
	}
	else if (group == 0x0028)
	{
	    switch(element)
	    {
		case 0x0004: /* Photometric interpretation */
		    readString(photometric, length);
		    printf("Photometric interpretation: %s\n", photometric);
		    break;
		case 0x0006: /* Planar 0 or 1 */
		    readBytes((char *) &planar, 2, ENDIAN);
		    printf("Planar mode: %d\n", planar);
		    break;
		case 0x0010: /* Rows */
		    readBytes((char *) &rows, 2, ENDIAN);
		    break;
		case 0x0011: /* Columns */
		    readBytes((char *) &columns, 2, ENDIAN);
		    printf("Image size: %d x %d\n", columns, rows);
		    break;
		case 0x0008: /* # of frames */
		    readString(stringBuffer, length); /* integer string */
		    sscanf(stringBuffer, "%d", &frameCount);
		    printf("%d frames in multi-frame image\n", frameCount);
		    break;
		case 0x1211: /* Large Red Palette CLUT Data */
		    warn = 0;
		    for (i = 0; i < 256; i++)
		    {
			readBytes(stringBuffer, 2, ENDIAN);
			if (stringBuffer[0] != stringBuffer[1]) warn = 1;
			palette[i][0] = stringBuffer[1];
		    }
		    break;
		case 0x1212: /* Large Green Palette CLUT Data */
		    for (i = 0; i < 256; i++)
		    {
			readBytes(stringBuffer, 2, ENDIAN);
			if (stringBuffer[0] != stringBuffer[1]) warn = 1;
			palette[i][1] = stringBuffer[1];
		    }
		    break;
		case 0x1213: /* Large Blue Palette CLUT Data */
		    for (i = 0; i < 256; i++)
		    {
			readBytes(stringBuffer, 2, ENDIAN);
			if (stringBuffer[0] != stringBuffer[1]) warn = 1;
			palette[i][2] = stringBuffer[1];
		    }
		    printf("Color palette loaded\n");
		    if (warn == 1) printf("WARNING: LUT invalid\n");
		    break;
		default: fseek(inFile, length, SEEK_CUR);
	    }
	}
	else if ((group == 0x7FE0) && (element == 0x0010))
	{
	    if ((encodedPixels == FALSE) && (length == 0xffffffff))
	    {
		printf("Unknown length given but pixels not encoded\n");
		return;
	    }
	    else if ((encodedPixels == TRUE) && (length != 0xffffffff))
	    {
		printf("Length specification not valid for encoded pixels\n");
		return;
	    }
	}
	else fseek(inFile, length, SEEK_CUR);
    } while ((group != 0x7FE0) || (element != 0x0010));

    printf("Do you want to display the image [N/Y]? ==> ");
    gets(stringBuffer);
    if ((strlen(stringBuffer) == 0) ||
	((strlen(stringBuffer)>0) && (toupper(stringBuffer[0]) != 'Y')))
    {
	if (fclose(inFile) != 0) printf("Error closing file %s", fileName);
	return;
    }

    if (frameCount > 1)
    {
	do {
	    printf("How many frames do you want to display [%d]? ==> ",
		frameCount);
	    gets(stringBuffer);
	    if (strlen(stringBuffer) == 0) displayCount = frameCount;
	    else
	    {
		sscanf(stringBuffer, "%d", &displayCount);
		if ((displayCount < 0) || (displayCount > frameCount))
		{
		    printf("Pick a number from 1 to %d\n", frameCount);
		    displayCount = 0;
		}
	    }
	} while (displayCount == 0);
    }
    else displayCount = 1;

    /*
     * At this point, we have scanned the entire IOD, pulling off
     * whatever attributes are interesting for display purposes.  We
     * have encountered the 0x7FE0 group, meaning that pixels will follow.
     * There are several ways that these pixels might have been written
     * into the IOD:
     *
     * (1) The (7FE0,0010) tag has a length that is NOT 0xFFFFFFFF.
     *     This should NOT happen for an XA IOD, as it means that the
     *     pixel data is not encoded (XA IOD's require lossless JPEG).
     *     For the DISC'95 demo, ultrasound IOD's are on the disc, and
     *     are NOT compressed -- they use this format.  Frames can be
     *     read by reading <columns> X <rows> bytes (one frame's worth).
     *
     * (2) The (7FE0,0010) tag has a length that IS 0xFFFFFFFF.  The next
     *     attribute should be (FFFE,E000), a basic offset table.  It
     *     can have a length field of 0x00000000 (meaning no offset table
     *     information is given), or may contain concatenated 32-bit
     *     unsigned integer values that are byte offsets to the first
     *     byte of the Item Tag of the first fragment for each frame in
     *     the Sequence of Items. (See Part 5, Section A.4).  As of the
     *     beta version of the software, the offset table IS included.
     *
     * (3) Given the situation in #2 above, the pixel data can be
     *     included in one or more fragments.  As of the beta release
     *     of the software, the encoded pixel data is written as one
     *     fragment per frame.  It is this situation that is implemented
     *     by the code that follows.  Implementatin of JPEG decompression
     *     is vendor-specific, and not shown here.  Public domain software
     *     for lossless JPEG compression and decompression is included in
     *     the software distribution to assist those vendors who do not
     *     yet have experience in this area.
     */

    if (encodedPixels == TRUE)
    {
	/* Check to see if there is an offset table, as expected */
	readTag(&group, &element, &length);
	if ((group != 0xFFFE) || (element != 0xE000))
	{
	    printf("Offset table not found when expected\n");
	    return;
	}
	if (length == 0)
	{
	    printf("Offset table has zero length (cannot handle this!)\n");
	    return;
	}
    
	if ((frameCount*4) != length)
	{
	    printf("Offset table size (%lu) does not match # of frames (%d)\n",
		length, frameCount);
	    return;
	} 

	for (i = 0; i < frameCount; i++)
	    readBytes((char *) &offsetTable[i], 4, ENDIAN);
    }
    pixelStart = ftell(inFile);

    /*
     * At this point, you have everything you need to start displaying
     * images.  Insert your code here to loop thru the image data,
     * performing decompression if needed, and moving the image data
     * to the frame buffer.  A simple loop is shown here that simply
     * counts the number of bytes of compressed data for each frame.
     */

#ifdef __BORLANDC__
    if (strcmp(photometric, "MONOCHROME2") == 0)
    {
	graphicsDriver = installuserdriver("svga256", NULL);
	graphicsMode = 3; /* 800 x 600 x 8bit */
    }
    else
    {
	graphicsDriver = installuserdriver("svgatc", NULL);
	graphicsMode = 3; /* 640 x 480 x 24bit */
    }
    initgraph(&graphicsDriver, &graphicsMode, "");
    grErr = graphresult();
    if (grErr != grOk)
    {
	printf("Unable to display graphics:%d\n", grErr);
	if (fclose(inFile) != 0) printf("Error closing file %s", fileName);
	return;
    }

    if (strcmp(photometric, "MONOCHROME2") == 0)
    {
	struct REGPACK reg;
	int j;

	for (j=0; j<256; j++) palette[j][0]=palette[j][1]=palette[j][2] = j/4;
	reg.r_ax = 0x1012;
	reg.r_bx = 0;
	reg.r_cx = 256;
	reg.r_es = FP_SEG(palette);
	reg.r_dx = FP_OFF(palette);
	intr(0x10,&reg);
    }
#endif

    /*
     * NOTE: In the display loops that follow, implementors should
     * use the value of the Frame Time (0018,1063) to control the rate
     * of looping.  This assumes (as is true for this demo) that a
     * constant time will be used between frames.  In the loops below,
     * comment lines show where you might insert statements to control
     * the looping rate -- starting a count-down timer at the top of
     * the loop, and waiting for the timer to count down to zero at the
     * bottom of the loop.  This assumes, of course, that you could display
     * the images faster than the "Frame Time" allows (not true when trying
     * to read from CD-ROM and/or decompress the images.  But, once images
     * have been read once, and the program starts displaying from RAM,
     * then these timing parameters become essential.
     *
     * Also note that the "fseek()" statements below are not needed,
     * since the file is already correctly positioned at the necessary
     * frame, and the loops show sequential reading.  The code is written
     * this way to show the correct computation of the fseek() parameters
     * to randomly seek to any frame.
     */

    if (encodedPixels == TRUE)
    {
	for (i = 0; i < displayCount; i++)
	{
	    /* >> Insert code here to start count-down timer << */
	    fseek(inFile, pixelStart + offsetTable[i], SEEK_SET);
	    readTag(&group, &element, &length);
	    if ((group != 0xFFFE) || (element != 0xE000))
	    {
		printf("Pixel data fragment not found when expected\n");
		return;
	    }
	    rowNumber = 0; /* Get ready to receive decompressed rows */
#ifndef __BORLANDC__
	    printf("Frame %d: %lu bytes\n", i + 1, length);
#endif
	    decompFrame();
	    /* >> Insert code here to wait for timer to run down to zero << */
	}
    }
    else if (strcmp(photometric, "MONOCHROME2") == 0)
    {
	for (i = 0; i < displayCount; i++)
	{
	    /* >> Insert code here to start count-down timer << */
	    fseek(inFile, pixelStart + ((unsigned long) i *
		(unsigned long) rows * (unsigned long) columns), SEEK_SET);
	    for (rowNumber = 0; rowNumber < (int) rows; rowNumber++)
	    {
		readBytes((char *) pixelBuffer, columns, NO_ENDIAN);
#ifdef __BORLANDC__
		for (j = 0; j < columns; j++)
		    putpixel(j+150, rowNumber+42, pixelBuffer[j]);
#endif
	    }
	    /* >> Insert code here to wait for timer to run down to zero << */
	}
    }
    else if (strcmp(photometric, "RGB") == 0)
    {
	for (i = 0; i < displayCount; i++)
	{
	    /* >> Insert code here to start count-down timer << */
	    fseek(inFile, pixelStart + ((unsigned long) (i *  3) *
		(unsigned long) rows * (unsigned long) columns), SEEK_SET);
	    for (rowNumber = 0; rowNumber < (int) rows; rowNumber++)
	    {
		readBytes((char *) pixelBuffer, columns * 3, NO_ENDIAN);
		for (j = 0; j < (int) columns; j++)
		{
#ifdef __BORLANDC__
		    setrgbpalette(1026, pixelBuffer[j*3],
			pixelBuffer[(j*3) + 1], pixelBuffer[(j*3) + 2]);
		    putpixel(j, rowNumber, 1026);
#endif
		}
	    }
	    /* >> Insert code here to wait for timer to run down to zero << */
	}
    }
    else if (strcmp(photometric, "PALETTE COLOR") == 0)
    {
	for (i = 0; i < displayCount; i++)
	{
	    /* >> Insert code here to start count-down timer << */
	    fseek(inFile, pixelStart + ((unsigned long) i *
		(unsigned long) rows * (unsigned long) columns), SEEK_SET);
	    for (rowNumber = 0; rowNumber < (int) rows; rowNumber++)
	    {
		readBytes((char *) pixelBuffer, columns, NO_ENDIAN);
		for (j = 0; j < (int) columns; j++)
		{
#ifdef __BORLANDC__
		    setrgbpalette(1026, palette[pixelBuffer[j]][0],
			palette[pixelBuffer[j]][1],
			palette[pixelBuffer[j]][2]);
		    putpixel(j, rowNumber, 1026);
#endif
		}
	    }
	    /* >> Insert code here to wait for timer to run down to zero << */
	}
    }
    else
    {
	printf("Unknown photometric interpretation: %s\n", photometric);
    }
#ifdef __BORLANDC__
    closegraph();
#endif

    if (fclose(inFile) != 0)
	printf("Error closing file %s", oneRecord->imageInfo.fileID);
}


/************************************************************************
 *									*
 * The parseDicomInfo() routine reads the introductory information from	*
 * DICOMDIR.  
 *									*
 ************************************************************************/

UnsignedLong parseDicomInfo(void)
{
    struct metaInfoStruct metaInfo;	/* File Meta Information */
    unsigned short consistency,		/* File consistency flag */
	    group,			/* Group data from tag */
	    element;			/* Element data from tag */
    UnsignedLong length,		/* Length data from tag */
	    firstPatientOffset,		/* Start of 1st patient's directory */
	    lastPatientOffset;		/* Start of last patient's directory */
    char descriptorFileName[VR_CS_LENGTH + 1];
    char descriptorFormat[VR_CS_LENGTH + 1];

    readFileMetaInfo(&metaInfo);
    if (strcmp(metaInfo.transferSyntaxUID, LITTLE_ENDIAN_EXPLICIT_UID) != 0)
	badFile("DICOMDIR for APL-XA-BCCD must be \"Little Endian Explicit\"");
    explicit_VR = TRUE;

    readTag(&group, &element, &length);
    if ((group != 4) || (element != 0x1130)) badFile("Invalid File-set ID");
    if (length > 0) readString(filesetID, length);
    filesetID[length] = '\0'; /* Terminate the string */
    readTag(&group, &element, &length);
    if (group != 4) badFile("Invalid directory information");
    if (element == 0x1141)
    {
	readString(descriptorFileName, length);
        descriptorFileName[length] = '\0';
	readTag(&group, &element, &length);
	if ((group != 4) || (element != 0x1142))
	    badFile("Fileset Descriptor Format is missing (0004,1142)");
	readString(descriptorFormat, length);
        descriptorFormat[length] = '\0';
	/* Add your code here to display the descriptor file */

	readTag(&group, &element, &length);
    }
    if (group != 4) badFile("Invalid directory information");
    while (element != 0x1220) /* Read until directory record sequence found */
    {
	switch(element)
	{
	    case 0x1200:
		readBytes((char *) &firstPatientOffset, length, ENDIAN);
		break;
	    case 0x1202:
		readBytes((char *) &lastPatientOffset, length, ENDIAN);
		break;
	    case 0x1212:
		readBytes((char *) &consistency, length, ENDIAN);
		break;
	    default: badFile("Invalid Directory Information");
	}
	readTag(&group, &element, &length);
	if (group != 4) badFile("Invalid directory information");
    }

    /* Length of zero indicates that no records are in the Directory */
    if (length == 0) return 0;
    if (consistency != 0) badFile("File-set may contain inconsistencies");
    return firstPatientOffset;
}


/************************************************************************
 *									*
 * The parseDir() routine is called to extract information from a	*
 * single DICOMDIR directory record and save it into the normalized	*
 * data structure.							*
 *									*
 ************************************************************************/

UnsignedLong parseDir(
    UnsignedLong thisDir,	/* Directory record to be scanned */
    UnsignedLong *lowerDir,	/* Return the pointer to next level down */
    char *recordType)		/* Type of directory record to be scanned */
{
    UnsignedLong length, dirLength, lengthStack[MAX_DEPTH];
    unsigned short group, element, inUseFlag;
    char msg[80];
    char patientName[VR_PN_LENGTH+1];
    char patientID[VR_LO_LENGTH + 1];
    char dirType[VR_CS_LENGTH + 1];
    int i, itemDepth = 0;
    struct dirRecord *newRecord;
    static struct dirRecord *lastRecord;

    while(thisDir != 0)
    {
	fseek(inFile, thisDir, SEEK_SET);
	/* Read the Item Data Element */
	readTag(&group, &element, &dirLength);
	if ((group != 0xfffe) || (element != 0xe000))
	    badFile("Invalid or missing Item Data Element");
    
	/* Read the offset to the next directory record */
	readTag(&group, &element, &length);
	readBytes((char *) &thisDir, length, ENDIAN);
    
	/* Read the Record in Use flag */
	readTag(&group, &element, &length);
	readBytes((char *) &inUseFlag, length, ENDIAN);
	/* If directory record not "in use", skip it */
	if (inUseFlag != 0xffff) continue; /* Go round the while loop again */
	
	/* Read the offset to the next lower directory level */
	readTag(&group, &element, &length);
	readBytes((char *) lowerDir, length, ENDIAN);
    
	/* Read the directory type */
	memset(dirType, 0, sizeof(dirType));
	readTag(&group, &element, &length);
	readString(dirType, length);
	/* If not interested in this type of directory record, ignore it */
	if (strcmp(dirType, recordType) != 0) continue;

	/*
	 * Now loop through the items in the directory record.  If a
	 * valid length was given in the data item element, there will be
	 * no final item tag, and the length can be used as a guide to
	 * how much data to read.  If the length was 0xFFFFFFFF, then the
	 * length is unknown, and a final item tag marks the end of useful
	 * data.  So, this loop will continue until the dirLength value
	 * is decremented to 0, or until a terminating tag is found (in
	 * which case dirLength is forced to 0 and the loop terminates.
	 * In this manner, either type of data grouping is handled
	 * correctly (unknown length terminated with item tag, or specified
	 * length).
	 */
	while (dirLength > 0)
	{
	    readTag(&group, &element, &length);
	    switch(group)
	    {
		case 0x0004:
		    switch(element)
		    {
			case 0x1500: /* (0004,1500) */
			    if (strcmp(dirType, "PATIENT") == 0)
				readString(thisRec.patientInfo.fileID, length);
			    else if (strcmp(dirType, "STUDY") == 0)
				readString(thisRec.studyInfo.fileID, length);
			    else if (strcmp(dirType, "SERIES") == 0)
				readString(thisRec.seriesInfo.fileID, length);
			    else if (strcmp(dirType, "IMAGE") == 0)
				readString(thisRec.imageInfo.fileID, length);
			    break;
			case 0x1510: /* (0004,1510) */
			    if (strcmp(dirType, "PATIENT") == 0)
				readBytes(thisRec.patientInfo.SOP_ClassUID,
				    length, NO_ENDIAN);
			    else if (strcmp(dirType, "STUDY") == 0)
				readBytes(thisRec.studyInfo.SOP_ClassUID,
				    length, NO_ENDIAN);
			    if (strcmp(dirType, "SERIES") == 0)
				readBytes(thisRec.seriesInfo.SOP_ClassUID,
				    length, NO_ENDIAN);
			    if (strcmp(dirType, "IMAGE") == 0)
				readBytes(thisRec.imageInfo.SOP_ClassUID,
				    length, NO_ENDIAN);
			    break;
			case 0x1511: /* (0004,1511) */
			    if (strcmp(dirType, "PATIENT") == 0)
				readBytes(thisRec.patientInfo.SOP_InstanceUID,
				    length, NO_ENDIAN);
			    else if (strcmp(dirType, "STUDY") == 0)
				readBytes(thisRec.studyInfo.SOP_InstanceUID,
				    length, NO_ENDIAN);
			    if (strcmp(dirType, "SERIES") == 0)
				readBytes(thisRec.seriesInfo.SOP_InstanceUID,
				    length, NO_ENDIAN);
			    if (strcmp(dirType, "IMAGE") == 0)
				readBytes(thisRec.imageInfo.SOP_InstanceUID,
				    length, NO_ENDIAN);
			    break;
			default: fseek(inFile, length, SEEK_CUR);
		    }
		    break; /* End of group 0x0004 */
		case 0x0008:
		    switch(element)
		    {
			case 0x0020: /* (0008,0020) */
			    if (length > 0)
				readString(thisRec.studyInfo.studyDate, length);
			    break;
			case 0x0030: /* (0008,0030) */
			    if (length > 0)
				readString(thisRec.studyInfo.studyTime, length);
			    break;
			case 0x0060: /* (0008,0060) */
			    if (length > 0)
				readString(thisRec.seriesInfo.modality, length);
			    break;
			case 0x0080: /* (0008,0080) */
			    if (length > 0)
				readString(thisRec.seriesInfo.institutionName,
				    length);
			    break;
			case 0x0081: /* (0008,0081) */
			    if (length > 0)
				readString(
				    thisRec.seriesInfo.institutionAddress,
				    length);
			    break;
			case 0x1030: /* (0008,1030) */
			     if (length > 0)
				 readString(thisRec.studyInfo.studyDescription,
				     length);
			     break;
			case 0x1050: /* (0008,1050) */
			    if (length > 0)
				readString(thisRec.seriesInfo.performingMD,
				    length);
			    break;
			default: fseek(inFile, length, SEEK_CUR);
		    }
		    break; /* end of group 0x0008 */
		case 0x0010:
		    switch(element)
		    {
			case 0x0010: /* (0010,0010) */
			    if (length > 0)
				readString(thisRec.patientInfo.patientName,
				    length);
			    break;
			case 0x0020: /* (0010,0020) */
			    if (length > 0)
				readString(thisRec.patientInfo.patientID,
				    length);
			    break;
			default: fseek(inFile, length, SEEK_CUR);
		    }
		    break; /* end of group 0x0010 */
		case 0x0020:
		    switch(element)
		    {
			case 0x0011: /* (0020,0011) */
			    if (length > 0)
				readString(thisRec.seriesInfo.seriesNumber,
				    length);
			    break;
			case 0x0013: /* (0020,0013) */
			    if (length > 0)
				readString(thisRec.imageInfo.imageNumber,
				    length);
			    break;
			case 0x0020: /* (0020,0020) */
			    if (length > 0)
				readBytes(thisRec.seriesInfo.seriesInstanceUID,
				    length, NO_ENDIAN);
			    break;
			case 0x0021: /* (0020,0021) */
			    if (length > 0)
				readString(thisRec.seriesInfo.seriesNumber,
				    length);
			    break;
			default: fseek(inFile, length, SEEK_CUR);
		    }
		    break; /* end of group 0x0020 */
		case 0x0050:
		    switch(element)
		    {
			case 0x0004: /* (0050,0004) */
			    if (length > 0)
				readString(thisRec.imageInfo.calibrationObject,
				    length);
			    break;
			default: fseek(inFile, length, SEEK_CUR);
		    }
		    break; /* end of group 0x0050 */
		case 0x7FE0:
		    switch(element)
		    { /* +++ NOTE: Should check Icon size and read that */
			case 0x0010: /* (7FE0,0010) */
#ifndef __MSDOS__
			    for (i = 0; i < 128; i++)
				readBytes((char *) thisRec.imageInfo.icon[i],
				    128, NO_ENDIAN);
			    break;
#endif
			default: fseek(inFile, length, SEEK_CUR);
		    }
		    break; /* end of group 0x7FE0 */
		case 0x0088:
		    switch(element)
		    {
			case 0x0200: /* (0088,0200) */
			    if (itemDepth == MAX_DEPTH)
				badFile("Too many nested items");
			    lengthStack[itemDepth++] = dirLength;
			    dirLength = length;
			    break;
			default: fseek(inFile, length, SEEK_CUR);
		    }
		    break; /* end of group 0x0088 */
		case 0xFFFE:
		    switch(element)
		    {
			case 0xE000: /* (FFFE,E000) */
			    if (itemDepth == MAX_DEPTH)
				badFile("Too many nested items");
			    lengthStack[itemDepth++] = dirLength;
			    dirLength = length;
			    break;
			case 0xE00D: /* (FFFE,E00D) */
			case 0xE0DD: /* (FFFE,E0DD) */
			    if (itemDepth == 0) dirLength = 0;
			    else dirLength = lengthStack[--itemDepth];
			    break;
			default: fseek(inFile, length, SEEK_CUR);
		    }
		    break; /* end of group 0xFFFE */
		default: fseek(inFile, length, SEEK_CUR);
	    } /* switch(group) */
	} /* while (dirLength > 0) */

	/* If this is an image record, add it to the linked list */
	if (strcmp(dirType, "IMAGE") == 0)
	{
	    /* Allocate memory for the new entry into the list */
	    newRecord = (struct dirRecord *) malloc(sizeof(struct dirRecord));
	    if (newRecord == (struct dirRecord *) NULL)
		badFile("Out of memory");
	    /* Copy the flattened version we have created into the new entry */
	    memcpy((char *)newRecord,(char *)&thisRec,sizeof(struct dirRecord));
	    /* If pointer to the list is NULL, this must be the first entry */
	    if (dirList == (struct dirRecord *) NULL) dirList = newRecord;
	    /* Otherwise, add it to the end of the list */
	    else lastRecord->nextRecord = newRecord;
	    /* Remember last entry in the list for next time we extend it */
	    lastRecord = newRecord;
	    /* Finally, set the new entry to show it is the end of the list */
	    newRecord->nextRecord = (struct dirRecord *) NULL;
	}
	return thisDir; /* Return pointer to next dir record at the level */
    } /* while(thisDir != 0) */

    /*
     * If the while() loop falls thru to here, no more valid directory
     * records were found at this level.
     */
    return 0;
}


/************************************************************************
 *									*
 * The parseDicomDir() is a simple control loop that traverses the	*
 * DICOMDIR directory tree creating a flattened record.  The nested	*
 * while() loop implements a depth-first search of the directory tree,	*
 * extracting first patient information, then study information, series	*
 * information, and image information.					*
 *									*
 ************************************************************************/

void parseDicomDir(UnsignedLong firstPatientPtr)
{
    UnsignedLong studyPtr;
    UnsignedLong seriesPtr;
    UnsignedLong imagePtr;
    UnsignedLong nextDir;

    while (firstPatientPtr != 0) 
    {
	memset((char *) &thisRec.patientInfo, 0, sizeof(struct patientData));
	firstPatientPtr = parseDir(firstPatientPtr, &studyPtr, "PATIENT");
	while (studyPtr != 0)
	{
	    memset((char *) &thisRec.studyInfo, 0, sizeof(struct studyData));
	    studyPtr = parseDir(studyPtr, &seriesPtr, "STUDY");
	    while (seriesPtr != 0)
	    {
		memset((char *)&thisRec.seriesInfo,0,sizeof(struct seriesData));
		seriesPtr = parseDir(seriesPtr, &imagePtr, "SERIES");
		while (imagePtr != 0)
		{
		    memset((char *) &thisRec.imageInfo, 0,
			sizeof(struct imageData));
		    /*imagePtr=parseDir(imagePtr,(UnsignedLong *) NULL,"IMAGE");*/
		    imagePtr=parseDir(imagePtr, &nextDir,"IMAGE");
		} /* image loop */
	    } /* series loop */
	} /* study loop */
    } /* patient loop */
}

/************************************************************************
 *									*
 * The main() part of the program is written as "bare-bones" code that	*
 * is intended to show how the subroutines are used to manipulate the	*
 * data in the DICOM file set.  The nested "while" loop simply has the	*
 * user select a patient, then a study for that patient, a series in	*
 * that study, then images within the series.  At any time, a NULL	*
 * answer pops up to the next higher level of selection (except a NULL	*
 * response to patient selection, which would then exit the program).	*
 *									*
 ************************************************************************/

int main()
{
    UnsignedLong firstPatient;
    struct dirRecord *oneRecord;
    char fileName[32];

    thisEndian = checkEndian(); /* note endian status for later */
    initGUI(); /* Initialize the Graphic User Interface (if any) */

    strcpy(fileName, CDROM_PATH);
    strcat(fileName, DICOMDIR_NAME);
    if ((inFile=fopen(fileName, "rb")) == NULL) /* open DICOMDIR */
	badFile("Unable to open DICOMDIR file");

    firstPatient = parseDicomInfo();	/* Read the information from DICOMDIR */
    if (firstPatient == 0) badFile("DICOMDIR is empty");

    parseDicomDir(firstPatient); /* Parse out the directory information */
    if (fclose(inFile) != 0) badFile("Error closing DICOMDIR file");

    initList(); /* Get ready to make the selection list */

    oneRecord = dirList; /* Loop thru the list of directory records */
    while (oneRecord != (struct dirRecord *) NULL)
    {
	addToList(oneRecord); /* Add each record to the selection list */
	oneRecord = oneRecord->nextRecord;
    }

    /*
     * This version of the control loop allows the user to repeatedly
     * make selections from the list that has been constructed.  For each
     * selection made, a call is made to the routines to allow display
     * of the image.  If during the selection routine, the user requests
     * to exit program the selection routine returns NULL.  For a
     * demonstration at a show, this option would probably not be offered
     * (don't want to allow the demo software to be exited so easily).
     */

    do {
	oneRecord = selectFromList();
	if (oneRecord != (struct dirRecord *) NULL) displayImage(oneRecord);
    } while (oneRecord != (struct dirRecord *) NULL);

    printf("Good-bye\n");
    return 0;
}


