/*************************************************************************
 *************************************************************************
 **									**
 **	 		Copyright (c) 1994, 1995 			**
 **	    Brown University Institute for Medical Computing		**
 **	      and the American College of Cardiology (ACC)		**
 **									**
 ** The software and support documentation for the American College of	**
 ** Cardiology's 1995 demonstration of DICOM interchange media were	**
 ** developed at the Brown University Institute for Medical Computing.	**
 **									** 
 ** THIS SOFTWARE IS MADE AVAILABLE "AS IS", AND NEITHER THE AMERICAN	**
 ** COLLEGE OF CARDIOLOGY (ACC) NOR BROWN UNIVERSITY MAKE ANY WARRANTY	**
 ** ABOUT THE SOFTWARE, ITS PERFORMANCE, ITS MERCHANTABILITY OR FITNESS	**
 ** FOR ANY PARTICULAR USE, FREEDOM FROM DEFECTS, OR ITS CONFORMANCE TO	**
 ** ANY SPECIFICATION.  THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE	**
 ** OF THE SOFTWARE IS WITH THE USER.					**
 **									** 
 ** Copyright of the software and supporting documentation is jointly	**
 ** owned by the ACC and Brown University, and free access is hereby	**
 ** granted as a license to use this software, copy this software and	**
 ** prepare derivative works based upon this software.  However, any	**
 ** distribution of the source code or executable versions of this	**
 ** software, or of the	 supporting documentation (or of derivative	**
 ** works of the software or supporting documentation) must include	**
 ** this copyright notice and explanation.				**
 **									** 
 ** Author: Jonathan L. Elion MD, FACC					**
 **									**
 ************************************************************************* 
 *************************************************************************/ 

/************************************************************************
 *									*
 * This module implements a dictionary useful for accessing DICOM	*
 * messages.  The "dictionary" consists of a large table of DICOM tags	*
 * (group and element) and the associated Value Representation (VR).	*
 * This allows routines in the source code to call message-sending	*
 * routines without having to worry about how the actual data is	*
 * handled.  In essence, the dictionary implements the implicit aspect	*
 * of VR's.								*
 *									*
 * This dictionary should be nearly complete, as it was derived	from	*
 * a machine-readable version of Part 6.  The description field is 	*
 * included in the table so that parsing and printing routines can	*
 * annotate listings.							*
 *									*
 ************************************************************************/

#include "acc95.h"
#include <stdlib.h>

struct dictStruct dict[] = {
    {0x0002, 0x0000, VR_UL, "Group Length"},
    {0x0002, 0x0001, VR_OB, "Meta Information Version Number"}, 
    {0x0002, 0x0002, VR_UI, "SOP Class UID"},
    {0x0002, 0x0003, VR_UI, "SOP Instance UID"},
    {0x0002, 0x0010, VR_UI, "Transfer Syntax UID"},
    {0x0002, 0x0012, VR_UI, "Implementation Class UID"},
    {0x0002, 0x0013, VR_SH, "Meta Information Version Name"},
    {0x0002, 0x0016, VR_AE, "Meta Information Entity Title"},
    {0x0002, 0x0100, VR_UI, "Private Information Creator UID"},
    {0x0002, 0x0102, VR_OB, "Private Information"},

    {0x0004, 0x1130, VR_CS, "File-set ID"},
    {0x0004, 0x1141, VR_CS, "File ID of File-set Descriptor"},
    {0x0004, 0x1142, VR_CS, "Format of File-set Descriptor File"},
    {0x0004, 0x1200, VR_UL, "Offset of 1st Dir Record of Root Dir Entity"},
    {0x0004, 0x1202, VR_UL, "Offset of Last Dir Record of Root Dir Entity"},
    {0x0004, 0x1212, VR_US, "File-set Consistency Flag"},
    {0x0004, 0x1220, VR_SQ, "Directory Record Sequence"},
    {0x0004, 0x1400, VR_UL, "Offset of next directry record"},
    {0x0004, 0x1410, VR_US, "Record in use flag"},
    {0x0004, 0x1420, VR_UL, "Offset of referenced lower-level dir entity"},
    {0x0004, 0x1430, VR_CS, "Directory record type"},
    {0x0004, 0x1500, VR_CS, "Referenced file ID"},
    {0x0004, 0x1510, VR_UI, "Referenced SOP Class UID in file"},
    {0x0004, 0x1511, VR_UI, "Referenced SOP Instance UID in file"},
    {0x0004, 0x1512, VR_UI, "Referenced Transfer Syntax UID in file"},
    {0x0004, 0x1600, VR_UL, "Number of Referencs"},

    {0x0008, 0x0000, VR_UL, "Group Length"},
    {0x0008, 0x0005, VR_CS, "Specific Character Set"},
    {0x0008, 0x0008, VR_CS, "Image Type"},
    {0x0008, 0x0012, VR_DA, "Instance Creation Date"},
    {0x0008, 0x0013, VR_TM, "Instance Creation Time"},
    {0x0008, 0x0014, VR_UI, "Instance Creator UID"},
    {0x0008, 0x0016, VR_UI, "SOP Class UID"},
    {0x0008, 0x0018, VR_UI, "SOP Instance UID"},
    {0x0008, 0x0020, VR_DA, "Study Date"},
    {0x0008, 0x0021, VR_DA, "Series Date"},
    {0x0008, 0x0022, VR_DA, "Acquisition Date"},
    {0x0008, 0x0023, VR_DA, "Image Date"},
    {0x0008, 0x0024, VR_DA, "Overlay Date"},
    {0x0008, 0x0025, VR_DA, "Curve Date"},
    {0x0008, 0x0030, VR_TM, "Study Time"},
    {0x0008, 0x0031, VR_TM, "Series Time"},
    {0x0008, 0x0032, VR_TM, "Acquisition Time"},
    {0x0008, 0x0033, VR_TM, "Image Time"},
    {0x0008, 0x0034, VR_TM, "Overlay Time"},
    {0x0008, 0x0035, VR_TM, "Curve Time"},
    {0x0008, 0x0042, VR_CS, "Nuclear Medicine Series Type"},
    {0x0008, 0x0050, VR_SH, "Accession Number"},
    {0x0008, 0x0052, VR_CS, "Query/Retrieve Level"},
    {0x0008, 0x0054, VR_AE, "Retrieve AE Title"},
    {0x0008, 0x0058, VR_UI, "Failed SOP Instance UID List"},
    {0x0008, 0x0060, VR_CS, "Modality"},
    {0x0008, 0x0064, VR_CS, "Conversion Type"},
    {0x0008, 0x0070, VR_LO, "Manufacturer"},
    {0x0008, 0x0080, VR_LO, "Institution Name"},
    {0x0008, 0x0081, VR_ST, "Institution Address"},
    {0x0008, 0x0082, VR_SQ, "Institution Code Sequence"},
    {0x0008, 0x0090, VR_PN, "Referring Physician's Name"},
    {0x0008, 0x0092, VR_ST, "Referring Physician's Address"},
    {0x0008, 0x0094, VR_SH, "Referring Physician's Telephone Numbers"},
    {0x0008, 0x0100, VR_SH, "Code Value"},
    {0x0008, 0x0102, VR_SH, "Coding Scheme Designator"},
    {0x0008, 0x0104, VR_LO, "Code Meaning"},
    {0x0008, 0x1010, VR_SH, "Station Name"},
    {0x0008, 0x1030, VR_LO, "Study Description"},
    {0x0008, 0x1032, VR_SQ, "Procedure Code Sequence"},
    {0x0008, 0x103E, VR_LO, "Series Description"},
    {0x0008, 0x1040, VR_LO, "Institutional Department Name"},
    {0x0008, 0x1048, VR_PN, "Physician of Record"},
    {0x0008, 0x1050, VR_PN, "Performing Physician's Name"},
    {0x0008, 0x1060, VR_PN, "Name of Physician(s) Reading Study"},
    {0x0008, 0x1070, VR_PN, "Operators' Name"},
    {0x0008, 0x1080, VR_LO, "Admitting Diagnoses Description"},
    {0x0008, 0x1084, VR_SQ, "Admitting Diagnosis Code Sequence"},
    {0x0008, 0x1090, VR_LO, "Manufacturer's Model Name"},
    {0x0008, 0x1100, VR_SQ, "Referenced Results Sequence"},
    {0x0008, 0x1110, VR_SQ, "Referenced Study Sequence"},
    {0x0008, 0x1111, VR_SQ, "Referenced Study Component Sequence"},
    {0x0008, 0x1115, VR_SQ, "Referenced Series Sequence"},
    {0x0008, 0x1120, VR_SQ, "Referenced Patient Sequence"},
    {0x0008, 0x1125, VR_SQ, "Referenced Visit Sequence"},
    {0x0008, 0x1130, VR_SQ, "Referenced Overlay Sequence"},
    {0x0008, 0x1140, VR_SQ, "Referenced Image Sequence"},
    {0x0008, 0x1145, VR_SQ, "Referenced Curve Sequence"},
    {0x0008, 0x1150, VR_UI, "Referenced SOP Class UID"},
    {0x0008, 0x1155, VR_UI, "Referenced SOP Instance UID"},
    {0x0008, 0x1160, VR_IS, "Referenced Frame Number"},
    {0x0008, 0x2111, VR_ST, "Derivation Description"},
    {0x0008, 0x2112, VR_SQ, "Source Image Sequence"},
    {0x0008, 0x2120, VR_SH, "Stage Name"},
    {0x0008, 0x2122, VR_IS, "Stage Number"},
    {0x0008, 0x2124, VR_IS, "Number of Stages"},
    {0x0008, 0x2128, VR_IS, "View Number"},
    {0x0008, 0x2129, VR_IS, "Number of Event Timers"},
    {0x0008, 0x212A, VR_IS, "Number of Views in Stage"},
    {0x0008, 0x2130, VR_DS, "Event Elapsed Time(s)"},
    {0x0008, 0x2132, VR_LO, "Event Timer Name(s)"},
    {0x0008, 0x2142, VR_IS, "Start Trim"},
    {0x0008, 0x2143, VR_IS, "Stop Trim"},
    {0x0008, 0x2144, VR_IS, "Recommended Display Frame Rate"},
    {0x0008, 0x2200, VR_CS, "Transducer Position"},
    {0x0008, 0x2204, VR_CS, "Transducer Orientation"},
    {0x0008, 0x2208, VR_CS, "Anatomic Structure"},
    {0x0008, 0x2218, VR_SQ, "Anatomic Region Sequence"},
    {0x0008, 0x2220, VR_SQ, "Anatomic Region Modifier Sequence"},
    {0x0008, 0x2228, VR_SQ, "Primary Anatomic Structure Sequence"},
    {0x0008, 0x2230, VR_SQ, "Primary Anatomic Structure Modifier Sequence"},
    {0x0008, 0x2240, VR_SQ, "Transducer Position Sequence"},
    {0x0008, 0x2242, VR_SQ, "Transducer Position Modifier Sequence"},
    {0x0008, 0x2244, VR_SQ, "Transducer Orientation Sequence"},
    {0x0008, 0x2246, VR_SQ, "Transducer Orientation Modifier Seq"},

    {0x0010, 0x0000, VR_UL, "Group Length"},
    {0x0010, 0x0010, VR_PN, "Patient's Name"},
    {0x0010, 0x0020, VR_LO, "Patient ID"},
    {0x0010, 0x0021, VR_LO, "Issuer of Patient ID"},
    {0x0010, 0x0030, VR_DA, "Patient's Birth Date"},
    {0x0010, 0x0032, VR_TM, "Patient's Birth Time"},
    {0x0010, 0x0040, VR_CS, "Patient's Sex"},
    {0x0010, 0x0050, VR_SQ, "Patient's Insurance Plan Code Sequence"},
    {0x0010, 0x1000, VR_LO, "Other Patient IDs"},
    {0x0010, 0x1001, VR_PN, "Other Patient Names"},
    {0x0010, 0x1005, VR_PN, "Patient's Birth Name"},
    {0x0010, 0x1010, VR_AS, "Patient's Age"},
    {0x0010, 0x1020, VR_DS, "Patient's Size"},
    {0x0010, 0x1030, VR_DS, "Patient's Weight"},
    {0x0010, 0x1040, VR_LO, "Patient's Address"},
    {0x0010, 0x1060, VR_PN, "Patient's Mother's Birth Name"},
    {0x0010, 0x1080, VR_LO, "Military Rank"},
    {0x0010, 0x1081, VR_LO, "Branch of Service"},
    {0x0010, 0x1090, VR_LO, "Medical Record Locator"},
    {0x0010, 0x2000, VR_LO, "Medical Alerts"},
    {0x0010, 0x2110, VR_LO, "Contrast Allergies"},
    {0x0010, 0x2150, VR_LO, "Country of Residence"},
    {0x0010, 0x2152, VR_LO, "Region of Residence"},
    {0x0010, 0x2154, VR_SH, "Patient's Telephone Numbers"},
    {0x0010, 0x2160, VR_SH, "Ethnic Group"},
    {0x0010, 0x2180, VR_SH, "Occupation"},
    {0x0010, 0x21A0, VR_CS, "Smoking Status"},
    {0x0010, 0x21B0, VR_LT, "Additional Patient History"},
    {0x0010, 0x21C0, VR_US, "Pregnancy Status"},
    {0x0010, 0x21D0, VR_DA, "Last Menstrual Date"},
    {0x0010, 0x21F0, VR_LO, "Patient's Religious Preference"},
    {0x0010, 0x4000, VR_LT, "Patient Comments"},

    {0x0018, 0x0000, VR_UL, "Group Length"},
    {0x0018, 0x0010, VR_LO, "Contrast/Bolus Agent"},
    {0x0018, 0x0012, VR_SQ, "Contrast/Bolus Agent Sequence"},
    {0x0018, 0x0014, VR_SQ, "Contrast/Bolus Administration Route Sequence"},
    {0x0018, 0x0015, VR_CS, "Body Part Examined"},
    {0x0018, 0x0020, VR_CS, "Scanning Sequence"},
    {0x0018, 0x0021, VR_CS, "Sequence Variant"},
    {0x0018, 0x0022, VR_CS, "Scan Options"},
    {0x0018, 0x0023, VR_CS, "MR Acquisition Type"},
    {0x0018, 0x0024, VR_SH, "Sequence Name"},
    {0x0018, 0x0025, VR_CS, "Angio Flag"},
    {0x0018, 0x0029, VR_SQ, "Interventional Drug Sequence"},
    {0x0018, 0x002A, VR_SQ, "Additional Drug Sequence"},
    {0x0018, 0x0030, VR_LO, "Radionuclide"},
    {0x0018, 0x0031, VR_LO, "Radiopharmaceutical"},
    {0x0018, 0x0032, VR_DS, "Energy Window Centerline"},
    {0x0018, 0x0033, VR_DS, "Energy Window Total Width"},
    {0x0018, 0x0034, VR_LO, "Intervention Drug Name"},
    {0x0018, 0x0035, VR_TM, "Intervention Drug Start Time"},
    {0x0018, 0x0036, VR_SQ, "Intervention Therapy Sequence"},
    {0x0018, 0x0037, VR_CS, "Therapy Type"},
    {0x0018, 0x0038, VR_CS, "Intervention Status"},
    {0x0018, 0x0040, VR_IS, "Cine Rate"},
    {0x0018, 0x0050, VR_DS, "Slice Thickness"},
    {0x0018, 0x0060, VR_DS, "KVP"},
    {0x0018, 0x0070, VR_IS, "Counts Accumulated"},
    {0x0018, 0x0071, VR_CS, "Acquisition Termination Condition"},
    {0x0018, 0x0072, VR_DS, "Effective Series Duration"},
    {0x0018, 0x0080, VR_DS, "Repetition Time"},
    {0x0018, 0x0081, VR_DS, "Echo Time"},
    {0x0018, 0x0082, VR_DS, "Inversion Time"},
    {0x0018, 0x0083, VR_DS, "Number of Averages"},
    {0x0018, 0x0084, VR_DS, "Imaging Frequency"},
    {0x0018, 0x0085, VR_SH, "Imaged Nucleus"},
    {0x0018, 0x0086, VR_IS, "Echo Number(s)"},
    {0x0018, 0x0087, VR_DS, "Magnetic Field Strength"},
    {0x0018, 0x0088, VR_DS, "Spacing Between Slices"},
    {0x0018, 0x0089, VR_IS, "Number of Phase Encoding Steps"},
    {0x0018, 0x0090, VR_DS, "Data Collection Diameter"},
    {0x0018, 0x0091, VR_IS, "Echo Train Length"},
    {0x0018, 0x0093, VR_DS, "Percent Sampling"},
    {0x0018, 0x0094, VR_DS, "Percent Phase Field of View"},
    {0x0018, 0x0095, VR_DS, "Pixel Bandwidth"},
    {0x0018, 0x1000, VR_LO, "Device Serial Number"},
    {0x0018, 0x1004, VR_LO, "Plate ID"},
    {0x0018, 0x1010, VR_LO, "Secondary Capture Device ID"},
    {0x0018, 0x1012, VR_DA, "Date of Secondary Capture"},
    {0x0018, 0x1014, VR_TM, "Time of Secondary Capture"},
    {0x0018, 0x1016, VR_LO, "Secondary Capture Device Manufacturer"},
    {0x0018, 0x1018, VR_LO, "Secondary Capture Device Manufact Model Name"},
    {0x0018, 0x1019, VR_LO, "Secondary Capture Device Software Version(s)"},
    {0x0018, 0x1020, VR_LO, "Software Version(s)"},
    {0x0018, 0x1022, VR_SH, "Video Image Format Acquired"},
    {0x0018, 0x1023, VR_LO, "Digital Image Format Acquired"},
    {0x0018, 0x1030, VR_LO, "Protocol Name"},
    {0x0018, 0x1040, VR_LO, "Contrast/Bolus Route"},
    {0x0018, 0x1041, VR_DS, "Contrast/Bolus Volume"},
    {0x0018, 0x1042, VR_TM, "Contrast/Bolus Start Time"},
    {0x0018, 0x1043, VR_TM, "Contrast/Bolus Stop Time"},
    {0x0018, 0x1044, VR_DS, "Contrast/Bolus Total Dose"},
    {0x0018, 0x1045, VR_IS, "Syringe counts"},
    {0x0018, 0x1046, VR_DS, "Contrast Flow Rate(s)"},
    {0x0018, 0x1047, VR_DS, "Contrast Flow Duration(s)"},
    {0x0018, 0x1048, VR_CS, "Contrast/Bolus Intredient"},
    {0x0018, 0x1049, VR_DS, "Contrast/Bolus Intredient Concentration"},
    {0x0018, 0x1050, VR_DS, "Spatial Resolution"},
    {0x0018, 0x1060, VR_DS, "Trigger Time"},
    {0x0018, 0x1061, VR_LO, "Trigger Source or Type"},
    {0x0018, 0x1062, VR_IS, "Nominal Interval"},
    {0x0018, 0x1063, VR_DS, "Frame Time"},
    {0x0018, 0x1064, VR_LO, "Framing Type"},
    {0x0018, 0x1065, VR_DS, "Frame Time Vector"},
    {0x0018, 0x1066, VR_DS, "Frame Delay"},
    {0x0018, 0x1070, VR_LO, "Radionuclide Route"},
    {0x0018, 0x1071, VR_DS, "Radionuclide Volume"},
    {0x0018, 0x1072, VR_TM, "Radionuclide Start Time"},
    {0x0018, 0x1073, VR_TM, "Radionuclide Stop Time"},
    {0x0018, 0x1074, VR_DS, "Radionuclide Total Dose"},
    {0x0018, 0x1080, VR_CS, "Beat Rejection Flag"},
    {0x0018, 0x1081, VR_IS, "Low R-R Value"},
    {0x0018, 0x1082, VR_IS, "High R-R Value"},
    {0x0018, 0x1083, VR_IS, "Intervals Acquired"},
    {0x0018, 0x1084, VR_IS, "Intervals Rejected"},
    {0x0018, 0x1085, VR_LO, "PVC Rejection"},
    {0x0018, 0x1086, VR_IS, "Skip Beats"},
    {0x0018, 0x1088, VR_IS, "Heart Rate"},
    {0x0018, 0x1090, VR_IS, "Cardiac Number of Images"},
    {0x0018, 0x1094, VR_IS, "Trigger Window"},
    {0x0018, 0x1100, VR_DS, "Reconstruction Diameter"},
    {0x0018, 0x1110, VR_DS, "Distance Source to Detector"},
    {0x0018, 0x1111, VR_DS, "Distance Source to Patient"},
    {0x0018, 0x1114, VR_DS, "Estimated Radiographic Magnification Factor"},
    {0x0018, 0x1120, VR_DS, "Gantry/Detector Tilt"},
    {0x0018, 0x1130, VR_DS, "Table Height"},
    {0x0018, 0x1131, VR_DS, "Table Traverse"},
    {0x0018, 0x1134, VR_CS, "Table Motion"},
    {0x0018, 0x1135, VR_DS, "Table Vertical Increment"},
    {0x0018, 0x1136, VR_DS, "Table Lateral Increment"},
    {0x0018, 0x1137, VR_DS, "Table Longitudinal Increment"},
    {0x0018, 0x1138, VR_DS, "Table Angle"}, /* corrected from 0018,1137 */
    {0x0018, 0x1140, VR_CS, "Rotation Direction"},
    {0x0018, 0x1141, VR_DS, "Angular Position"},
    {0x0018, 0x1142, VR_DS, "Radial Position"},
    {0x0018, 0x1143, VR_DS, "Scan Arc"},
    {0x0018, 0x1144, VR_DS, "Angular Step"},
    {0x0018, 0x1145, VR_DS, "Center of Rotation Offset"},
    {0x0018, 0x1146, VR_DS, "Rotation Offset"},
    {0x0018, 0x1147, VR_CS, "Field of View Shape"},
    {0x0018, 0x1149, VR_IS, "Field of View Dimension(s)"},
    {0x0018, 0x1150, VR_IS, "Exposure Time"},
    {0x0018, 0x1151, VR_IS, "X-ray Tube Current"},
    {0x0018, 0x1152, VR_IS, "Exposure"},
    {0x0018, 0x1154, VR_DS, "Average Pulse Width"},
    {0x0018, 0x1155, VR_CS, "Radiation Setting"},
    {0x0018, 0x115A, VR_CS, "Radiation Mode"},
    {0x0018, 0x115E, VR_DS, "Image Area Dose Produce"},
    {0x0018, 0x1160, VR_SH, "Filter Type"},
    {0x0018, 0x1161, VR_LO, "Type of Filters"},
    {0x0018, 0x1162, VR_DS, "Intensifier Size"},
    {0x0018, 0x1164, VR_DS, "Imager Pixel Spacing"},
    {0x0018, 0x1166, VR_CS, "Grid"},
    {0x0018, 0x1170, VR_IS, "Generator Power"},
    {0x0018, 0x1180, VR_SH, "Collimator/grid Name"},
    {0x0018, 0x1181, VR_CS, "Collimator Type"},
    {0x0018, 0x1182, VR_IS, "Focal Distance"},
    {0x0018, 0x1183, VR_DS, "X Focus Center"},
    {0x0018, 0x1184, VR_DS, "Y Focus Center"},
    {0x0018, 0x1190, VR_DS, "Focal Spot(s)"},
    {0x0018, 0x1200, VR_DA, "Date of Last Calibration"},
    {0x0018, 0x1201, VR_TM, "Time of Last Calibration"},
    {0x0018, 0x1210, VR_SH, "Convolution Kernel"},
    {0x0018, 0x1242, VR_IS, "Actual Frame Duration"},
    {0x0018, 0x1243, VR_IS, "Count Rate"},
    {0x0018, 0x1250, VR_SH, "Receiving Coil"},
    {0x0018, 0x1251, VR_SH, "Transmitting Coil"},
    {0x0018, 0x1260, VR_SH, "Plate Type"},
    {0x0018, 0x1261, VR_LO, "Phosphor Type"},
    {0x0018, 0x1300, VR_IS, "Scan Velocity"},
    {0x0018, 0x1301, VR_CS, "Whole Body Technique"},
    {0x0018, 0x1302, VR_IS, "Scan Length"},
    {0x0018, 0x1310, VR_US, "Acquisition Matrix"},
    {0x0018, 0x1312, VR_CS, "Phase Encoding Direction"},
    {0x0018, 0x1314, VR_DS, "Flip Angle"},
    {0x0018, 0x1315, VR_CS, "Variable Flip Angle Flag"},
    {0x0018, 0x1316, VR_DS, "SAR"},
    {0x0018, 0x1318, VR_DS, "dB/dt"},
    {0x0018, 0x1400, VR_LO, "Acquisition Device Processing Description"},
    {0x0018, 0x1401, VR_LO, "Acquisition Device Processing Code"},
    {0x0018, 0x1402, VR_CS, "Cassette Orientation"},
    {0x0018, 0x1403, VR_CS, "Cassette Size"},
    {0x0018, 0x1404, VR_US, "Exposures on Plate"},
    {0x0018, 0x1405, VR_IS, "Relative X-ray Exposure"},
    {0x0018, 0x1500, VR_CS, "Positioner Motion"},
    {0x0018, 0x1510, VR_DS, "Positioner Primary Angle"},
    {0x0018, 0x1511, VR_DS, "Positioner Secondary Angle"},
    {0x0018, 0x1520, VR_DS, "Positioner Primary Angle Increment"},
    {0x0018, 0x1521, VR_DS, "Positioner Secondary Angle Increment"},
    {0x0018, 0x1600, VR_CS, "Shutter Shape"},
    {0x0018, 0x1602, VR_IS, "Shutter Left Vertical Edge"},
    {0x0018, 0x1604, VR_IS, "Shutter Right Vertical Edge"},
    {0x0018, 0x1606, VR_IS, "Shutter Upper Vertical Edge"},
    {0x0018, 0x1608, VR_IS, "Shutter Lower Vertical Edge"},
    {0x0018, 0x1610, VR_IS, "Center of Circular Shutter"},
    {0x0018, 0x1612, VR_IS, "Radius of Circular Shutter"},
    {0x0018, 0x1620, VR_IS, "Vertices of the Polygonal Shutter"},
    {0x0018, 0x1700, VR_CS, "Collimator Shape"},
    {0x0018, 0x1702, VR_IS, "Collimator Left Vertical Edge"},
    {0x0018, 0x1704, VR_IS, "Collimator Right Vertical Edge"},
    {0x0018, 0x1706, VR_IS, "Collimator Upper Horizontal Edge"},
    {0x0018, 0x1708, VR_IS, "Collimator Lower Horizontal Edge"},
    {0x0018, 0x1710, VR_IS, "Center of Circular Collimator"},
    {0x0018, 0x1712, VR_IS, "Radius of Circular Collimator"},
    {0x0018, 0x1720, VR_IS, "Vertices of the Polygonal Collimator"},
    {0x0018, 0x5000, VR_SH, "Output Power"},
    {0x0018, 0x5010, VR_LO, "Transducer"},
    {0x0018, 0x5012, VR_DS, "Focus Depth"},
    {0x0018, 0x5020, VR_LO, "Preprocessing Function"},
    {0x0018, 0x5021, VR_LO, "Postprocessing Function"},
    {0x0018, 0x5022, VR_DS, "Mechanical Index"},
    {0x0018, 0x5024, VR_DS, "Thermal Index"},
    {0x0018, 0x5026, VR_DS, "Cranial Thermal Index"},
    {0x0018, 0x5027, VR_DS, "Soft Tissue Thermal Index"},
    {0x0018, 0x5028, VR_DS, "Soft Tissue-focus Thermal Index"},
    {0x0018, 0x5029, VR_DS, "Soft Tissue-surface Thermal Index"},
    {0x0018, 0x5050, VR_IS, "Depth of Scan Field"},
    {0x0018, 0x5100, VR_CS, "Patient Position"},
    {0x0018, 0x5101, VR_CS, "View Position"},
    {0x0018, 0x5210, VR_DS, "Image Transformation Matrix"},
    {0x0018, 0x5212, VR_DS, "Image Translation Vector"},
    {0x0018, 0x6000, VR_DS, "Sensitivity"},
    {0x0018, 0x6011, VR_SQ, "Sequence of Ultrasound Regions"},
    {0x0018, 0x6012, VR_US, "Region Spatial Format"},
    {0x0018, 0x6016, VR_UL, "Region Flags"},
    {0x0018, 0x6018, VR_UL, "Region Location Min X0"},
    {0x0018, 0x601A, VR_UL, "Region Location Min Y0"},
    {0x0018, 0x601C, VR_UL, "Region Location Max X1"},
    {0x0018, 0x601E, VR_UL, "Region Location Max Y1"},
    {0x0018, 0x6020, VR_SL, "Reference Pixel X0"},
    {0x0018, 0x6022, VR_SL, "Reference Pixel Y0"},
    {0x0018, 0x6024, VR_US, "Physical Units X Direction"},
    {0x0018, 0x6026, VR_US, "Physical Units Y Direction"},
    {0x0018, 0x6028, VR_FD, "Reference Pixel Physical Value X"},
    {0x0018, 0x602A, VR_FD, "Reference Pixel Physical Value Y"},
    {0x0018, 0x602C, VR_FD, "Physical Delta X"},
    {0x0018, 0x602E, VR_FD, "Physical Delta Y"},
    {0x0018, 0x6030, VR_UL, "Transducer Frequency"},
    {0x0018, 0x6031, VR_CS, "Transducer Type"},
    {0x0018, 0x6032, VR_UL, "Pulse Repetition Frequency"},
    {0x0018, 0x6034, VR_FD, "Doppler Correction Angle"},
    {0x0018, 0x6036, VR_FD, "Sterring Angle"},
    {0x0018, 0x6038, VR_UL, "Doppler Sample Volume X Position"},
    {0x0018, 0x603A, VR_UL, "Doppler Sample Volume Y Position"},
    {0x0018, 0x603C, VR_UL, "TM-Line Position X0"},
    {0x0018, 0x603E, VR_UL, "TM-Line Position Y0"},
    {0x0018, 0x6040, VR_UL, "TM-Line Position X1"},
    {0x0018, 0x6042, VR_UL, "TM-Line Position Y1"},
    {0x0018, 0x6044, VR_US, "Pixel Component Organization"},
    {0x0018, 0x6046, VR_UL, "Pixel Component Mask"},
    {0x0018, 0x6048, VR_UL, "Pixel Component Range Start"},
    {0x0018, 0x604A, VR_UL, "Pixel Component Range Stop"},
    {0x0018, 0x604C, VR_US, "Pixel Component Physical Units"},
    {0x0018, 0x604E, VR_US, "Pixel Component Data Type"},
    {0x0018, 0x6050, VR_UL, "Number of Table Break Points"},
    {0x0018, 0x6052, VR_UL, "Table of X Break Points"},
    {0x0018, 0x6054, VR_FD, "Table of Y Break Points"},
    {0x0018, 0x6056, VR_UL, "Number of Table Entries"},
    {0x0018, 0x6058, VR_UL, "Table of Pixel values"},
    {0x0018, 0x605A, VR_FL, "Table of Parameter values"},

    {0x0020, 0x0000, VR_UL, "Group Length"},
    {0x0020, 0x000D, VR_UI, "Study Instance UID"},
    {0x0020, 0x000E, VR_UI, "Series Instance UID"},
    {0x0020, 0x0010, VR_SH, "Study ID"},
    {0x0020, 0x0011, VR_IS, "Series Number"},
    {0x0020, 0x0012, VR_IS, "Acquisition Number"},
    {0x0020, 0x0013, VR_IS, "Image Number"},
    {0x0020, 0x0014, VR_IS, "Isotope Number"},
    {0x0020, 0x0015, VR_IS, "Phase Number"},
    {0x0020, 0x0016, VR_IS, "Interval Number"},
    {0x0020, 0x0017, VR_IS, "Time Slot Number"},
    {0x0020, 0x0018, VR_IS, "Angle Number"},
    {0x0020, 0x0020, VR_CS, "Patient Orientation"},
    {0x0020, 0x0022, VR_IS, "Overlay Number"},
    {0x0020, 0x0024, VR_IS, "Curve Number"},
    {0x0020, 0x0026, VR_IS, "LUT Number"},
    {0x0020, 0x0032, VR_DS, "Image Position (Patient)"},
    {0x0020, 0x0037, VR_DS, "Image Orientation (Patient)"},
    {0x0020, 0x0052, VR_UI, "Frame of Reference UID"},
    {0x0020, 0x0060, VR_CS, "Laterality"},
    {0x0020, 0x0100, VR_IS, "Temporal Position Identifier"},
    {0x0020, 0x0105, VR_IS, "Number of Temporal Positions"},
    {0x0020, 0x0110, VR_DS, "Temporal Resolution"},
    {0x0020, 0x1000, VR_IS, "Series in Study"},
    {0x0020, 0x1002, VR_IS, "Images in Acquisition"},
    {0x0020, 0x1004, VR_IS, "Acquisition in Study"},
    {0x0020, 0x1040, VR_LO, "Position Reference Indicator"},
    {0x0020, 0x1041, VR_DS, "Slice Location"},
    {0x0020, 0x1070, VR_IS, "Other Study Numbers"},
    {0x0020, 0x1200, VR_IS, "Number of Patient Related Studies"},
    {0x0020, 0x1202, VR_IS, "Number of Patient Related Series"},
    {0x0020, 0x1204, VR_IS, "Number of Patient Related Images"},
    {0x0020, 0x1206, VR_IS, "Number of Study Related Series"},
    {0x0020, 0x1208, VR_IS, "Number of Study Related Images"},
    {0x0020, 0x4000, VR_LT, "Image Comments"},

    {0x0028, 0x0000, VR_UL, "Group Length"},
    {0x0028, 0x0002, VR_US, "Samples per Pixel"},
    {0x0028, 0x0004, VR_CS, "Photometric Interpretation"},
    {0x0028, 0x0006, VR_US, "Planar Configuration"},
    {0x0028, 0x0008, VR_IS, "Number of Frames"},
    {0x0028, 0x0009, VR_AT, "Frame Increment Pointer"},
    {0x0028, 0x0010, VR_US, "Rows"},
    {0x0028, 0x0011, VR_US, "Columns"},
    {0x0028, 0x0012, VR_US, "Planes"},
    {0x0028, 0x0014, VR_US, "Color Image"},
    {0x0028, 0x0030, VR_DS, "Pixel"},
    {0x0028, 0x0031, VR_DS, "Zoom"},
    {0x0028, 0x0032, VR_DS, "Zoom"},
    {0x0028, 0x0034, VR_IS, "Pixel Aspect Ratio"},
    {0x0028, 0x0051, VR_CS, "Corrected Image"},
    {0x0028, 0x0100, VR_US, "Bits Allocated"},
    {0x0028, 0x0101, VR_US, "Bits Stored"},
    {0x0028, 0x0102, VR_US, "High Bit"},
    {0x0028, 0x0103, VR_US, "Pixel Representation"},
    {0x0028, 0x0106, VR_SS, "Smallest Image Pixel Value"},
    {0x0028, 0x0107, VR_SS, "Largest Image Pixel Value"},
    {0x0028, 0x0108, VR_SS, "Smallest Pixel Value in Series"},
    {0x0028, 0x0109, VR_SS, "Largest Pixel Value in Series"},
    {0x0028, 0x0110, VR_SS, "Smallest Image Pixel Value in Plane"},
    {0x0028, 0x0111, VR_SS, "Largest Image Pixel Value in Plane"},
    {0x0028, 0x0120, VR_SS, "Pixel Padding Value"},
    {0x0028, 0x1040, VR_CS, "Pixel Intensity Relationship"},
    {0x0028, 0x1050, VR_DS, "Window Center"},
    {0x0028, 0x1051, VR_DS, "Window Width"},
    {0x0028, 0x1052, VR_DS, "Rescale Intercept"},
    {0x0028, 0x1053, VR_DS, "Rescale Slope"},
    {0x0028, 0x1054, VR_LO, "Rescale type"},
    {0x0028, 0x1055, VR_LO, ") Window Center & Width Explanation"},
    {0x0028, 0x1101, VR_SS, "Red Palette Color Lookup Table Descriptor"},
    {0x0028, 0x1102, VR_SS, "Green Palette Color Lookup Table Descriptor"},
    {0x0028, 0x1103, VR_SS, "Blue Palette Color Lookup Table Descriptor"},
    {0x0028, 0x1111, VR_US, "Large Red Palette CLUT Descriptor"},
    {0x0028, 0x1112, VR_US, "Large Green Palette CLUT Descriptor"},
    {0x0028, 0x1113, VR_US, "Large Blue Palette CLUT Descriptor"},
    {0x0028, 0x1201, VR_SS, "Red Palette Color Lookup Table Data"},
    {0x0028, 0x1202, VR_SS, "Green Palette Color Lookup Table Data"},
    {0x0028, 0x1203, VR_SS, "Blue Palette Color Lookup Table Data"},
    {0x0028, 0x1211, VR_OW, "Large Red Palette CLUT Data"},
    {0x0028, 0x1212, VR_OW, "Large Green Palette CLUT Data"},
    {0x0028, 0x1213, VR_OW, "Large Blue Palette CLUT Data"},
    {0x0028, 0x1214, VR_UI, "Large Palette UID"},
    {0x0028, 0x1244, VR_US, "Preferred Playback Sequencing"},
    {0x0028, 0x2110, VR_US, "Lossy image procesing"},
    {0x0028, 0x3000, VR_SQ, "Modality LUT Sequence"},
    {0x0028, 0x3002, VR_SS, "LUT Descriptor"},
    {0x0028, 0x3003, VR_LO, "LUT Explanation"},
    {0x0028, 0x3004, VR_LO, "Modality LUT Type"},
    {0x0028, 0x3006, VR_SS, "LUT Data"},
    {0x0028, 0x3010, VR_SQ, "VOI LUT Sequence"},
    {0x0028, 0x5000, VR_SQ, "Bi-Plane Acquisition Sequence"},
    {0x0028, 0x6010, VR_US, "Representative Frame Number"},
    {0x0028, 0x6020, VR_US, "Frame Numbers of Interest (FOI)"},
    {0x0028, 0x6022, VR_LO, "Frame of Interest Description"},
    {0x0028, 0x6030, VR_US, "Mask Pointer(s)"},
    {0x0028, 0x6040, VR_US, "R Wave Pointer"},
    {0x0028, 0x6100, VR_SQ, "Mask Subtraction Sequence"},
    {0x0028, 0x6101, VR_CS, "Mask Operation"},
    {0x0028, 0x6102, VR_US, "Applicable Frame Range"},
    {0x0028, 0x6110, VR_US, "Mask Frame Numbers"},
    {0x0028, 0x6112, VR_US, "Contrast Frame Averaging"},
    {0x0028, 0x6114, VR_FL, "Mask Sub-pixel Shift"},
    {0x0028, 0x6120, VR_SS, "TID Offset"},
    {0x0028, 0x6190, VR_ST, "Mask Operation Explanation"},

    {0x0032, 0x0000, VR_UL, "Group Length"},
    {0x0032, 0x000A, VR_CS, "Study Status ID"},
    {0x0032, 0x000C, VR_CS, "Study Priority ID"},
    {0x0032, 0x0012, VR_LO, "Study ID Issuer"},
    {0x0032, 0x0032, VR_DA, "Study Verified Date"},
    {0x0032, 0x0033, VR_TM, "Study Verified Time"},
    {0x0032, 0x0034, VR_DA, "Study Read Date"},
    {0x0032, 0x0035, VR_TM, "Study Read Time"},
    {0x0032, 0x1000, VR_DA, "Scheduled Study Start Date"},
    {0x0032, 0x1001, VR_TM, "Scheduled Study Start Time"},
    {0x0032, 0x1010, VR_DA, "Scheduled Study Stop Date"},
    {0x0032, 0x1011, VR_TM, "Scheduled Study Stop Time"},
    {0x0032, 0x1020, VR_LO, "Scheduled Study Location"},
    {0x0032, 0x1021, VR_AE, "Scheduled Study Location AE Title(s)"},
    {0x0032, 0x1030, VR_LO, "Reason for Study"},
    {0x0032, 0x1032, VR_PN, "Requesting Physician"},
    {0x0032, 0x1033, VR_LO, "Requesting Service"},
    {0x0032, 0x1040, VR_DA, "Study Arrival Date"},
    {0x0032, 0x1041, VR_TM, "Study Arrival Time"},
    {0x0032, 0x1050, VR_DA, "Study Completion Date"},
    {0x0032, 0x1051, VR_TM, "Study Completion Time"},
    {0x0032, 0x1055, VR_CS, "Study Component Status ID"},
    {0x0032, 0x1060, VR_LO, "Requested Procedure Description"},
    {0x0032, 0x1064, VR_SQ, "Requested Procedure Code Sequence"},
    {0x0032, 0x1070, VR_LO, "Requested Contrast Agent"},
    {0x0032, 0x4000, VR_LT, "Study Comments"},

    {0x0038, 0x0000, VR_UL, "Group Length"},
    {0x0038, 0x0004, VR_SQ, "Referenced Patient Alias Sequence"},
    {0x0038, 0x0008, VR_CS, "Visit Status ID"},
    {0x0038, 0x0010, VR_LO, "Admission ID"},
    {0x0038, 0x0011, VR_LO, "Issuer of Admission ID"},
    {0x0038, 0x0016, VR_LO, "Route of Admissions"},
    {0x0038, 0x001A, VR_DA, "Scheduled Admission Date"},
    {0x0038, 0x001B, VR_TM, "Scheduled Admission Time"},
    {0x0038, 0x001C, VR_DA, "Scheduled Discharge Date"},
    {0x0038, 0x001D, VR_TM, "Scheduled Discharge Time"},
    {0x0038, 0x001E, VR_LO, "Scheduled Patient Institution Residence"},
    {0x0038, 0x0020, VR_DA, "Admitting Date"},
    {0x0038, 0x0021, VR_TM, "Admitting Time"},
    {0x0038, 0x0030, VR_DA, "Discharge Date"},
    {0x0038, 0x0032, VR_TM, "Discharge Time"},
    {0x0038, 0x0040, VR_LO, "Discharge Diagnosis Description"},
    {0x0038, 0x0044, VR_SQ, "Discharge Diagnosis Code Sequence"},
    {0x0038, 0x0050, VR_LO, "Special Needs"},
    {0x0038, 0x0300, VR_LO, "Current Patient Location"},
    {0x0038, 0x0400, VR_LO, "Patient's Institution Residence"},
    {0x0038, 0x0500, VR_LO, "Patient State"},
    {0x0038, 0x4000, VR_LT, "Visit Comments"},

    {0x0050, 0x0004, VR_CS, "Calibration Object"},
    {0x0050, 0x0010, VR_SQ, "Device Sequence"},
    {0x0050, 0x0012, VR_CS, "Device Type"},
    {0x0050, 0x0014, VR_DS, "Device Length"},
    {0x0050, 0x0016, VR_DS, "Device Diameter"},
    {0x0050, 0x0017, VR_CS, "Device Diameter Units"},
    {0x0050, 0x0018, VR_DS, "Device Volume"},
    {0x0050, 0x0019, VR_DS, "Inter-marker Distance"},
    {0x0050, 0x0020, VR_LO, "Device Description"},
    {0x0050, 0x0030, VR_SQ, "Coded Interventional Device Sequence"},

    {0x0088, 0x0000, VR_UL, "Group Length"},
    {0x0088, 0x0130, VR_SH, "Storage Media File-set ID"},
    {0x0088, 0x0140, VR_UI, "Storage Media File-set UID"},
    {0x0088, 0x0200, VR_SQ, "Icon Image"},
    {0x0088, 0x0904, VR_LO, "Topic Title"},
    {0x0088, 0x0906, VR_ST, "Topic Subject"},
    {0x0088, 0x0910, VR_LO, "Topic Author"},
    {0x0088, 0x0912, VR_LO, "Topic Key Words"},

    {0x2000, 0x0000, VR_UL, "Group Length"},
    {0x2000, 0x0010, VR_IS, "Number Of Copies"},
    {0x2000, 0x0020, VR_CS, "Print Priority"},
    {0x2000, 0x0030, VR_CS, "Medium Type"},
    {0x2000, 0x0040, VR_CS, "Film Destination"},
    {0x2000, 0x0050, VR_LO, "Film Session Label"},
    {0x2000, 0x0060, VR_IS, "Memory Allocation"},
    {0x2000, 0x0500, VR_SQ, "Referenced Film Box Sequence"},

    {0x2010, 0x0000, VR_UL, "Group Length"},
    {0x2010, 0x0010, VR_ST, "Image Display Format"},
    {0x2010, 0x0030, VR_CS, "Annotation Display Format ID"},
    {0x2010, 0x0040, VR_CS, "Film Orientation"},
    {0x2010, 0x0050, VR_CS, "Film Size ID"},
    {0x2010, 0x0060, VR_CS, "Magnification Type"},
    {0x2010, 0x0080, VR_CS, "Smoothing Type"},
    {0x2010, 0x0100, VR_CS, "Border Density"},
    {0x2010, 0x0110, VR_CS, "Empty Image Density"},
    {0x2010, 0x0120, VR_US, "Min Density"},
    {0x2010, 0x0130, VR_US, "Max Density"},
    {0x2010, 0x0140, VR_CS, "Trim"},
    {0x2010, 0x0150, VR_ST, "Configuration Information"},
    {0x2010, 0x0500, VR_SQ, "Referenced Film Session Sequence"},
    {0x2010, 0x0510, VR_SQ, "Referenced Image Box Sequence"},
    {0x2010, 0x0520, VR_SQ, "Referenced Basic Annotation Box Sequence"},

    {0x2020, 0x0000, VR_UL, "Group Length"},
    {0x2020, 0x0010, VR_US, "Image Position"},
    {0x2020, 0x0020, VR_CS, "Polarity"},
    {0x2020, 0x0030, VR_DS, "Requested Image Size"},
    {0x2020, 0x0110, VR_SQ, "Preformatted Grayscale Image Sequence"},
    {0x2020, 0x0111, VR_SQ, "Preformatted Color Image Sequence"},
    {0x2020, 0x0130, VR_SQ, "Referenced Image Overlay Box Sequence"},
    {0x2020, 0x0140, VR_SQ, "Referenced VOI LUT Box Sequence"},

    {0x2030, 0x0000, VR_UL, "Group Length"},
    {0x2030, 0x0010, VR_US, "Annotation Position"},
    {0x2030, 0x0020, VR_LO, "Text String"},

    {0x2040, 0x0000, VR_UL, "Group Length"},
    {0x2040, 0x0010, VR_SQ, "Referenced Overlay Plane Sequence"},
    {0x2040, 0x0011, VR_US, "Referenced Overlay Plane Groups"},
    {0x2040, 0x0060, VR_CS, "Overlay Magnification Type"},
    {0x2040, 0x0070, VR_CS, "Overlay Smoothing Type"},
    {0x2040, 0x0080, VR_CS, "Overlay Foreground Density"},
    {0x2040, 0x0090, VR_CS, "Overlay Mode"},
    {0x2040, 0x0100, VR_CS, "Threshold Density"},
    {0x2040, 0x0500, VR_SQ, "Referenced Image Box Sequence"},

    {0x2100, 0x0000, VR_UL, "Group Length"},
    {0x2100, 0x0020, VR_CS, "Execution Status"},
    {0x2100, 0x0030, VR_CS, "Execution Status Info"},
    {0x2100, 0x0040, VR_DA, "Creation Date"},
    {0x2100, 0x0050, VR_TM, "Creation Time"},
    {0x2100, 0x0070, VR_AE, "Originator"},
    {0x2100, 0x0500, VR_SQ, "Referenced Print Job Sequence"},

    {0x2110, 0x0000, VR_UL, "Group Length"},
    {0x2110, 0x0010, VR_CS, "Printer Status"},
    {0x2110, 0x0020, VR_CS, "Printer Status Info"},
    {0x2110, 0x0030, VR_LO, "Printer Name"},
    {0x2110, 0x0099, VR_SH, "Printer Queue ID"},

    {0x4008, 0x0000, VR_UL, "Group Length"},
    {0x4008, 0x0040, VR_SH, "Results ID"},
    {0x4008, 0x0042, VR_LO, "Results ID Issuer"},
    {0x4008, 0x0050, VR_SQ, "Referenced Interpretation Sequence"},
    {0x4008, 0x0100, VR_DA, "Interpretation Recorded Date"},
    {0x4008, 0x0101, VR_TM, "Interpretation Recorded Time"},
    {0x4008, 0x0102, VR_PN, "Interpretation Recorder"},
    {0x4008, 0x0103, VR_LO, "Reference to Recorded Sound"},
    {0x4008, 0x0108, VR_DA, "Interpretation Transcription Date"},
    {0x4008, 0x0109, VR_TM, "Interpretation Transcription Time"},
    {0x4008, 0x010A, VR_PN, "Interpretation Transcriber"},
    {0x4008, 0x010B, VR_ST, "Interpretation Text"},
    {0x4008, 0x010C, VR_PN, "Interpretation Author"},
    {0x4008, 0x0111, VR_SQ, "Interpretation Approver Sequence"},
    {0x4008, 0x0112, VR_DA, "Interpretation Approval Date"},
    {0x4008, 0x0113, VR_TM, "Interpretation Approval Time"},
    {0x4008, 0x0114, VR_PN, "Physician Approving Interpretation"},
    {0x4008, 0x0115, VR_LT, "Interpretation Diagnosis Description"},
    {0x4008, 0x0117, VR_SQ, "Diagnosis Code Sequence"},
    {0x4008, 0x0118, VR_SQ, "Results Distribution List Sequence"},
    {0x4008, 0x0119, VR_PN, "Distribution Name"},
    {0x4008, 0x011A, VR_LO, "Distribution Address"},
    {0x4008, 0x0200, VR_SH, "Interpretation ID"},
    {0x4008, 0x0202, VR_LO, "Interpretation ID Issuer"},
    {0x4008, 0x0210, VR_CS, "Interpretation Type ID"},
    {0x4008, 0x0212, VR_CS, "Interpretation Status ID"},
    {0x4008, 0x0300, VR_ST, "Impressions"},
    {0x4008, 0x4000, VR_ST, "Results Comments"},

    {0x7FE0, 0x0000, VR_UL, "Group Length"},
    {0x7FE0, 0x0010, VR_OB, "Pixel Data"},

    {0xFFFC, 0x0000, VR_UL, "Jeff's mystery Group Length"},
    {0xFFFC, 0xFFFC, VR_OB, "Data Set Trailing Padding"},

    {0xFFFE, 0xE000, VR_DATA, "Item"},
    {0xFFFE, 0xE00D, VR_DATA, "Item Delimitation Item"},
    {0xFFFE, 0xE0DD, VR_DATA, "Sequence Delimitation Item"},
    {0xffff, 0xffff, 0, ""}
};


/************************************************************************
 *									*
 * The dictLookup() routine gets passed the group and element number of	*
 * a DICOM message, and returns the dictionary entry.  A simple binary	*
 * search algorithm is used, examining a candidate _guess_ that is half	*
 * way between the search _top_ and search _bottom_.			*
 *									*
 ************************************************************************/

struct dictStruct *dictLookup(unsigned short group, unsigned short element)
{
    int	guess,			/* Element in table to be examined */
	bottom,			/* Bottom of the table search range */
	top;			/* Top of the table search range */
    static int dictCount = -1;	/* Number of elements in the table */

    if (dictCount == -1) /* If first time called, initialize count */
	for (dictCount=0; dict[dictCount].group != 0xffff; dictCount++);

    top = dictCount; /* Set top of search range to the top of the table */
    if ((dict[top].group == group)  && (dict[top].element == element))
	return(&dict[top]); /* Check to see if we are lucky */
    bottom = 0; /* Set bottom of search range to the bottom of the table */
    if ((dict[bottom].group == group)  && (dict[bottom].element == element))
	return(&dict[bottom]); /* Again, check to see if we are lucky */

    guess = (top + bottom) / 2; /* Guess at the middle of the search range */
    while ((guess != top) && (guess != bottom)) /* All of table searched */
    {
	if ((dict[guess].group == group)  && (dict[guess].element == element))
	    return(&dict[guess]); /* found it */
	if ((dict[guess].group < group) ||
	   ((dict[guess].group == group) && (dict[guess].element < element)))
	    bottom = guess; /* Looking too low -- reset bottom of range */
	else top = guess; /* Looking too high -- reset top of range */
	guess = (top + bottom) / 2; /* recompute middle for guess */
    }

    return((struct dictStruct *) NULL); /* Fell through loop -- not found */
}


